@@ -38,8 +38,8 @@
 #include "bhv_scan_field.h"
 #include "neck_scan_field.h"
 
-#include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
+#include <rcsc/common/logger.h>
 #include <rcsc/common/server_param.h>
 
 namespace rcsc {
@@ -55,8 +55,7 @@ bool
 Body_TurnToAngle::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
-                  "%s%d: Body_TurnToAngle"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": Body_TurnToAngle" );
 
     const SelfObject & self = agent->world().self();
 
@@ -80,8 +79,8 @@ bool
 Body_TurnToPoint::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
-                  "%s%d: Body_TurnToPoint"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": Body_TurnToPoint" );
+
     const SelfObject & self = agent->world().self();
 
     if ( ! self.posValid() )
@@ -119,6 +118,9 @@ Body_TurnToPoint::execute( PlayerAgent * agent )
 bool
 Body_TurnToBall::execute( PlayerAgent * agent )
 {
+    dlog.addText( Logger::ACTION,
+                  __FILE__": Body_TurnToBall" );
+
     if ( ! agent->world().ball().posValid() )
     {
         return false;
@@ -134,13 +136,53 @@ Body_TurnToBall::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+bool
+Body_TackleToPoint::execute( PlayerAgent * agent )
+{
+    dlog.addText( Logger::ACTION,
+                  __FILE__": Body_TackleToPoint" );
+
+    if ( agent->world().self().tackleProbability() < 0.01 )
+    {
+        return false;
+    }
+
+    AngleDeg target_rel_angle
+        = ( M_point - agent->world().ball().pos() ).th()
+        - agent->world().self().body();
+
+    if ( agent->config().version() < 12.0 )
+    {
+        if ( target_rel_angle.abs() < 90.0 )
+        {
+            return agent->doTackle( ServerParam::i().maxTacklePower() );
+        }
+        else if ( ServerParam::i().maxBackTacklePower() > 0.0 )
+        {
+            // backward case
+            return agent->doTackle( - ServerParam::i().maxBackTacklePower() );
+        }
+        return false;
+    }
+
+    // TODO: calculate the effective tackle power
+
+
+    return agent->doTackle( target_rel_angle.degree() );
+}
+
+/////////////////////////////////////////////////////////////////////
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 bool
 Neck_TurnToRelative::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
-                  "%s:%d: Neck_TurnToRelative"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": Neck_TurnToRelative" );
 
     return agent->doTurnNeck( M_angle_rel_to_body
                               - agent->world().self().neck() );
@@ -155,6 +197,9 @@ Neck_TurnToRelative::execute( PlayerAgent * agent )
 bool
 Neck_TurnToPoint::execute( PlayerAgent * agent )
 {
+    dlog.addText( Logger::ACTION,
+                  __FILE__": Neck_TurnToPoint" );
+
     const Vector2D next_pos = agent->effector().queuedNextMyPos();
     const AngleDeg next_body = agent->effector().queuedNextMyBody();
     const double next_view_width = agent->effector().queuedNextViewWidth().width() * 0.5;
@@ -177,8 +222,7 @@ Neck_TurnToPoint::execute( PlayerAgent * agent )
     }
 
     dlog.addText( Logger::ACTION,
-                  "%s:%d Neck_TurnToPoint. cannot turn neck to target points. scan"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": Neck_TurnToPoint. cannot turn neck to target points. scan" );
     Neck_ScanField().execute( agent );
     return true;
 }
@@ -193,8 +237,8 @@ bool
 Neck_TurnToBall::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
-                  "%s:%d: Neck_TurnToBall"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": Neck_TurnToBall" );
+
     if ( ! agent->world().ball().posValid() )
     {
         Neck_ScanField().execute( agent );
@@ -211,8 +255,8 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
     const double next_view_width = agent->effector().queuedNextViewWidth().width();
 
     dlog.addText( Logger::ACTION,
-                  "%s:%d: ball_next=(%.2f, %.2f), ball_rel_angle=%.1f, next_view=%.1f"
-                  ,__FILE__, __LINE__,
+                  __FILE__":%d: ball_next=(%.2f, %.2f), ball_rel_angle=%.1f, next_view=%.1f"
+                  , __LINE__,
                   ball_next.x, ball_next.y,
                   ball_rel_angle_next.degree(),
                   next_view_width );
@@ -226,14 +270,14 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
     }
 
     // ball is near, directly look the ball
-    if ( my_next.dist( ball_next ) < 20.0 )
+    if ( my_next.dist( ball_next ) < 15.0 ) // [200803]
     {
         agent->doTurnNeck( ball_rel_angle_next
                            - agent->world().self().neck() );
         return true;
     }
 
-    double angle_buf = std::max( 0.0, next_view_width * 0.5 - 15.0 );
+    double angle_buf = std::max( 0.0, next_view_width * 0.5 - 10.0 ); //[200803] 15.0
 
     AngleDeg left_rel_angle = ball_rel_angle_next - angle_buf;
     AngleDeg right_rel_angle = ball_rel_angle_next + angle_buf;
@@ -288,6 +332,9 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
 bool
 Bhv_BodyNeckToPoint::execute( PlayerAgent * agent )
 {
+    dlog.addText( Logger::ACTION,
+                  __FILE__": Bhv_BodyNeckToPoint" );
+
     if ( ! agent->world().self().posValid() )
     {
         return Bhv_ScanField().execute( agent );
@@ -299,7 +346,7 @@ Bhv_BodyNeckToPoint::execute( PlayerAgent * agent )
         + agent->world().self().vel();
 
     AngleDeg target_rel_angle
-        = (M_point - mynext).th()
+        = ( M_point - mynext ).th()
         - agent->world().self().body();
 
     // body & neck is already face to point.
@@ -357,6 +404,9 @@ Bhv_BodyNeckToPoint::execute( PlayerAgent * agent )
 bool
 Bhv_BodyNeckToBall::execute( PlayerAgent * agent )
 {
+    dlog.addText( Logger::ACTION,
+                  __FILE__": Bhv_BodyNeckToBall" );
+
     if ( agent->world().ball().posValid() )
     {
         Vector2D ballnext
@@ -380,16 +430,13 @@ bool
 Bhv_NeckBodyToPoint::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
-                  "%s:%d: Bhv_NeckBodyToPoint. to(%f, %f)"
-                  ,__FILE__, __LINE__,
+                  __FILE__": Bhv_NeckBodyToPoint.(%.1f, %.1f)",
                   M_point.x, M_point.y );
 
     // relative angle from my predicted pos & body angle to point
-    Vector2D mynext = agent->world().self().pos();
-    if ( agent->world().self().velValid() )
-    {
-        mynext += agent->world().self().vel();
-    }
+    Vector2D mynext
+        = agent->world().self().pos()
+        + agent->world().self().vel();
 
     AngleDeg target_rel_angle
         = ( M_point - mynext ).th()
@@ -403,7 +450,7 @@ Bhv_NeckBodyToPoint::execute( PlayerAgent * agent )
     //    return false;
     //}
 
-    // I can face to point only turn_neck.
+    // can face to point only by turn_neck.
     if ( ServerParam::i().minNeckAngle() + 5.0 < target_rel_angle.degree()
          && target_rel_angle.degree() < ServerParam::i().maxNeckAngle() - 5.0 )
     {
@@ -417,7 +464,7 @@ Bhv_NeckBodyToPoint::execute( PlayerAgent * agent )
                           agent->world().self().vel().r(),
                           agent->world().self().playerType().inertiaMoment() );
 
-    // can face only turn
+    // can face only by turn
     if ( target_rel_angle.abs() < max_turn )
     {
         dlog.addText( Logger::ACTION,
@@ -427,6 +474,7 @@ Bhv_NeckBodyToPoint::execute( PlayerAgent * agent )
         agent->setNeckAction( new Neck_TurnToRelative( 0.0 ) );
         return true;
     }
+
     // cannot face only turn
     // turn_neck is required.
     agent->doTurn( target_rel_angle );
@@ -458,8 +506,8 @@ bool
 Bhv_NeckBodyToBall::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
-                  "%s:%d: Bhv_NeckBodyToBall"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": Bhv_NeckBodyToBall" );
+
     if ( agent->world().ball().posValid() )
     {
         Vector2D ball_next
@@ -484,8 +532,7 @@ bool
 View_Wide::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
-                  "%s:%d: View_Wide"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": View_Wide" );
 
     return agent->doChangeView( ViewWidth::WIDE );
 }
@@ -500,12 +547,55 @@ bool
 View_Normal::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
-                  "%s:%d: View_Normal"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": View_Normal" );
 
     return agent->doChangeView( ViewWidth::NORMAL );
 }
 
 /////////////////////////////////////////////////////////////////////
 
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+bool
+Arm_PointToPoint::execute( PlayerAgent * agent )
+{
+    dlog.addText( Logger::ACTION,
+                  __FILE__": Arm_PointToPoint" );
+
+    if ( agent->world().self().armMovable() > 0 )
+    {
+        dlog.addText( Logger::ACTION,
+                      "Arm_PointToPoint. arm is not movable." );
+        return false;
+    }
+
+    return agent->doPointto( M_point.x, M_point.y );
+}
+
+/////////////////////////////////////////////////////////////////////
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+bool
+Arm_Off::execute( PlayerAgent * agent )
+{
+    dlog.addText( Logger::ACTION,
+                  __FILE__": Arm_Off" );
+
+    if ( agent->world().self().armMovable() > 0 )
+    {
+        dlog.addText( Logger::ACTION,
+                      "Arm_Off. arm is not movable." );
+        return false;
+    }
+
+    return agent->doPointtoOff();
+}
+
+/////////////////////////////////////////////////////////////////////
+
 }
