@@ -50,6 +50,13 @@ private:
     Vector2D M_target_point;
     //! ball first speed when ball is released
     double M_first_speed;
+
+    //! result ball position
+    Vector2D M_ball_result_pos;
+    //! result ball velocity
+    Vector2D M_ball_result_vel;
+    //! estimated kick step
+    int M_kick_step;
 public:
     /*!
       \brief construct with all parameters
@@ -60,6 +67,9 @@ public:
                       const double & first_speed )
         : M_target_point( target_point )
         , M_first_speed( first_speed )
+        , M_ball_result_pos( Vector2D::INVALIDATED )
+        , M_ball_result_vel( Vector2D::INVALIDATED )
+        , M_kick_step( 0 )
       { }
 
     /*!
@@ -69,6 +79,25 @@ public:
     */
     bool execute( PlayerAgent * agent );
 
+    /*!
+      \brief get the result ball position
+      \return ball position after kick
+     */
+    const
+    Vector2D & ballResultPos() const
+      {
+          return M_ball_result_pos;
+      }
+
+    /*!
+      \brief get the result ball velocity
+      \return ball velocity after kick
+     */
+    const
+    Vector2D & ballResultVel() const
+      {
+          return M_ball_result_vel;
+      }
 
 
     /*!
