@@ -37,9 +37,9 @@
 
 #include "body_stop_ball.h"
 
-#include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/logger.h>
+#include <rcsc/common/server_param.h>
 #include <rcsc/soccer_math.h>
 #include <rcsc/math_util.h>
 
@@ -54,12 +54,14 @@ const size_t Body_KickToRelative::DEFAULT_KICK_QUEUE_MAX = 5;
 bool
 Body_KickToRelative::execute( PlayerAgent * agent )
 {
+    const WorldModel & wm = agent->world();
+
     dlog.addText( Logger::ACTION,
                   "%s:%d: Body_KickToRelative dist=%.2f  rel_angle=%.1f"
                   ,__FILE__, __LINE__,
                   M_target_dist, M_target_angle_relative.degree() );
 
-    if ( ! agent->world().self().isKickable() )
+    if ( ! wm.self().isKickable() )
     {
         std::cerr << __FILE__ << ": " << __LINE__
                   << " not ball kickable!"
@@ -72,12 +74,12 @@ Body_KickToRelative::execute( PlayerAgent * agent )
 
     if ( M_stop_ball )
     {
-        if ( std::fabs( M_target_dist - agent->world().ball().distFromSelf() )
+        if ( std::fabs( M_target_dist - wm.ball().distFromSelf() )
              <= 0.1 )
         {
             AngleDeg ball_rang
-                = agent->world().ball().angleFromSelf()
-                - agent->world().self().body();
+                = wm.ball().angleFromSelf()
+                - wm.self().body();
             if ( ( M_target_angle_relative - ball_rang ).abs() < 4.0 )
             {
                 dlog.addText( Logger::ACTION,
@@ -88,12 +90,11 @@ Body_KickToRelative::execute( PlayerAgent * agent )
         }
     }
 
-
-    Vector2D required_accel( Vector2D::INVALIDATED );
+    Vector2D required_accel( 0.0, 0.0 );
 
     // simulate kick queue
-    if ( ! simulate( agent, &required_accel, false ) // simulate near side rotation
-         && ! simulate( agent, &required_accel, true ) ) // simulate far side rotation
+    if ( ! simulate( wm, false, &required_accel ) // simulate near side rotation
+         && ! simulate( wm, true, &required_accel ) ) // simulate far side rotation
     {
         dlog.addText( Logger::ACTION,
                       "%s:%d: failed. stop."
@@ -109,9 +110,9 @@ Body_KickToRelative::execute( PlayerAgent * agent )
                   ,__FILE__, __LINE__,
                   required_accel.x, required_accel.y,
                   accel_radius, accel_angle.degree(),
-                  agent->world().ball().distFromSelf(),
-                  agent->world().self().kickRate() * ServerParam::i().maxPower() );
-    if ( accel_radius < 0.01 )
+                  wm.ball().distFromSelf(),
+                  wm.self().kickRate() * ServerParam::i().maxPower() );
+    if ( accel_radius < 0.05 )
     {
         dlog.addText( Logger::ACTION,
                       "%s:%d: accell is very small. not needed to kick."
@@ -120,8 +121,8 @@ Body_KickToRelative::execute( PlayerAgent * agent )
         return false;
     }
 
-    return agent->doKick( accel_radius / agent->world().self().kickRate(),
-                          accel_angle - agent->world().self().body() );
+    return agent->doKick( accel_radius / wm.self().kickRate(),
+                          accel_angle - wm.self().body() );
 }
 
 /*-------------------------------------------------------------------*/
@@ -129,16 +130,14 @@ Body_KickToRelative::execute( PlayerAgent * agent )
 
 */
 bool
-Body_KickToRelative::simulate( const PlayerAgent * agent,
-                               Vector2D * required_accel,
-                               const bool reverse )
+Body_KickToRelative::simulate( const WorldModel & wm,
+                               const bool reverse,
+                               Vector2D * required_accel )
 {
     /*----------------------------------------------------------*/
     // now, we do NOT consider about opponent players
     // it may be necessary to check opponents in high level decisions.
 
-    const WorldModel & wm = agent->world();
-
     const double ROTATE_DIST
         = wm.self().playerType().playerSize()
         + wm.self().playerType().kickableMargin() * 0.5;
@@ -176,6 +175,8 @@ Body_KickToRelative::simulate( const PlayerAgent * agent,
         {
             // current iterator's value is next sub-target.
 
+            // TODO: opponent check
+
             Vector2D ball_rel_pos = ball_rpos + my_pos;
 
             my_pos += my_vel;
