@@ -42,12 +42,12 @@
 #include "body_hold_ball.h"
 
 #include <rcsc/player/interception.h>
-#include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
 #include <rcsc/player/debug_client.h>
 #include <rcsc/player/audio_sensor.h>
 #include <rcsc/player/say_message_builder.h>
 
+#include <rcsc/common/logger.h>
 #include <rcsc/common/server_param.h>
 #include <rcsc/geom/rect_2d.h>
 #include <rcsc/geom/sector_2d.h>
@@ -1249,7 +1249,7 @@ Body_Pass::verify_through_pass( const WorldModel & world,
         if ( (*it)->goalie() )
         {
             if ( target_point.absY() > ServerParam::i().penaltyAreaWidth() - 3.0
-                 || target_point.x < ServerParam::i().theirPenaltyAreaLine() + 2.0 )
+                 || target_point.x < ServerParam::i().theirPenaltyAreaLineX() + 2.0 )
             {
                 continue;
             }
