@@ -67,7 +67,7 @@ public:
      */
     struct PassRoute {
         PassType type_; //!< pass type id
-        const PlayerObject * receiver_; //< pointer to the receiver player
+        const PlayerObject * receiver_; //!< pointer to the receiver player
         Vector2D receive_point_; //!< estimated receive point
         double first_speed_; //!< ball first speed
         bool one_step_kick_; //!< true if ball reaches first speed only by one kick.
@@ -77,6 +77,7 @@ public:
           \brief construct with all member variables
           \param type pass type id
           \param receiver pointer to the receiver player
+          \param point pass receive point
           \param speed ball first speed
           \param one_step_kick true if ball reaches first speed only by one kick.
          */
