@@ -35,10 +35,10 @@
 
 #include "body_stop_ball.h"
 
-#include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
 #include <rcsc/player/debug_client.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/logger.h>
+#include <rcsc/common/server_param.h>
 
 #include <algorithm>
 
@@ -87,7 +87,7 @@ Body_StopBall::execute( PlayerAgent * agent )
     AngleDeg accel_angle;
     calcAccel( agent, &accel_radius, &accel_angle );
 
-    if ( accel_radius < 0.01 )
+    if ( accel_radius < 0.05 )
     {
         agent->doTurn( 0.0 );
         return false;
