@@ -42,9 +42,9 @@
 #include "body_hold_ball.h"
 #include "neck_turn_to_low_conf_teammate.h"
 
-#include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/logger.h>
+#include <rcsc/common/server_param.h>
 
 namespace rcsc {
 
@@ -118,7 +118,7 @@ IntentionKick::finished( const PlayerAgent * agent )
         }
         // check opponent goalie
         else if ( (*it)->goalie()
-                  && (*it)->pos().x > ServerParam::i().theirPenaltyAreaLine() + 1.5
+                  && (*it)->pos().x > ServerParam::i().theirPenaltyAreaLineX() + 1.5
                   && (*it)->pos().absY() < ServerParam::i().penaltyAreaHalfWidth() - 1.5
                   && (*it)->distFromBall() < ServerParam::i().catchAreaLength() )
         {
@@ -190,6 +190,8 @@ IntentionKick::execute( PlayerAgent * agent )
         // try one step
         if ( Body_KickTwoStep::simulate_one_kick
              ( &next_vel,
+               NULL,
+               NULL,
                target_rpos,
                M_first_speed,
                Vector2D(0.0, 0.0), // my current pos
@@ -242,7 +244,7 @@ IntentionKick::execute( PlayerAgent * agent )
                     &next_vel,
                     target_rpos,
                     M_first_speed,
-                    Vector2D(0.0, 0.0), // my current pos
+                    Vector2D( 0.0, 0.0 ), // my current pos
                     agent->world().self().vel(),
                     agent->world().self().body(),
                     agent->world().ball().rpos(),
@@ -283,7 +285,7 @@ IntentionKick::execute( PlayerAgent * agent )
         = ( M_target_point
             - ( agent->world().self().pos() + agent->world().self().vel() ) ).th();
 
-    if ( agent->world().getDirCount( target_angle ) == 0 )
+    if ( agent->world().dirCount( target_angle ) == 0 )
     {
          agent->setNeckAction( new Neck_TurnToLowConfTeammate() );
     }
