@@ -40,9 +40,9 @@
 #include "neck_scan_field.h"
 #include "neck_turn_to_ball_or_scan.h"
 
-#include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/logger.h>
+#include <rcsc/common/server_param.h>
 
 namespace rcsc {
 
@@ -62,33 +62,44 @@ Neck_TurnToGoalieOrScan::execute( PlayerAgent * agent )
     const PlayerObject * opp_goalie = wm.getOpponentGoalie();
 
     if ( ! opp_goalie
-         || opp_goalie->posCount() <= 2 )
+         || opp_goalie->posCount() <= std::max( 0, M_count_thr ) )
     {
-        dlog.addText( Logger::TEAM,
+        dlog.addText( Logger::ACTION,
                       "%s:%d: goalie not found, or already valid"
                       ,__FILE__, __LINE__ );
         return Neck_TurnToBallOrScan().execute( agent );
     }
 
+    if ( opp_goalie->isGhost()
+         && opp_goalie->posCount() >= 3 )
+    {
+        dlog.addText( Logger::ACTION,
+                      "%s:%d: detect ghost goalie"
+                      ,__FILE__, __LINE__ );
+        return Neck_TurnToBallOrScan().execute( agent );
+    }
+
     const ViewWidth vwidth = agent->effector().queuedNextViewWidth();
-    const double next_view_width = vwidth.getWidth();
+    const double next_view_width = vwidth.width();
     const Vector2D my_next = agent->effector().queuedNextMyPos();
     const AngleDeg my_next_body = agent->effector().queuedNextMyBody();
 
-    if ( ( ( opp_goalie->pos() - my_next ).th() - my_next_body ).abs()
+    const Vector2D goalie_next = opp_goalie->pos() + opp_goalie->vel();
+
+    if ( ( ( goalie_next - my_next ).th() - my_next_body ).abs()
          > ServerParam::i().maxNeckAngle() + next_view_width * 0.5 - 10.0 )
     {
-        dlog.addText( Logger::TEAM,
+        dlog.addText( Logger::ACTION,
                       "%s:%d: never face to goalie. next_width=%f"
                       ,__FILE__, __LINE__,
                       next_view_width );
         return Neck_TurnToBallOrScan().execute( agent );
     }
 
-    dlog.addText( Logger::TEAM,
+    dlog.addText( Logger::ACTION,
                   "%s:%d: neck to goalie"
                   ,__FILE__, __LINE__ );
-    return Neck_TurnToPoint( opp_goalie->pos() ).execute( agent );
+    return Neck_TurnToPoint( goalie_next ).execute( agent );
 }
 
 }
