@@ -38,8 +38,8 @@
 #include <rcsc/action/body_kick_one_step.h>
 
 #include <rcsc/player/player_agent.h>
-#include <rcsc/player/logger.h>
 #include <rcsc/player/interception.h>
+#include <rcsc/common/logger.h>
 #include <rcsc/common/server_param.h>
 #include <rcsc/math_util.h>
 
@@ -65,7 +65,7 @@ ShootTable::search( const PlayerAgent * agent )
 
     /////////////////////////////////////////////////////////////////////
     static const Rect2D
-        shootable_area( ServerParam::i().theirPenaltyAreaLine() - 5.0, // left
+        shootable_area( ServerParam::i().theirPenaltyAreaLineX() - 5.0, // left
                         -ServerParam::i().penaltyAreaHalfWidth(), // top
                         ServerParam::i().penaltyAreaLength() + 5.0, // length
                         ServerParam::i().penaltyAreaWidth() ); // width
@@ -236,7 +236,7 @@ bool
 ShootTable::canScore( const WorldModel & wm,
                       Shot * shot )
 {
-    static const double opp_x_thr = ServerParam::i().theirPenaltyAreaLine() - 5.0;
+    static const double opp_x_thr = ServerParam::i().theirPenaltyAreaLineX() - 5.0;
     static const double opp_y_thr = ServerParam::i().penaltyAreaHalfWidth();
 
     static const double player_max_speed
@@ -284,6 +284,14 @@ ShootTable::canScore( const WorldModel & wm,
         }
         else
         {
+#if 1
+            if ( (*it)->posCount() > 10
+                 || ( (*it)->isGhost() && (*it)->posCount() > 5 ) )
+            {
+                continue;
+            }
+#endif
+
             double cycle = ( std::ceil( util.getReachCycle( (*it)->pos(),
                                                             &((*it)->vel()),
                                                             NULL,
@@ -310,7 +318,7 @@ ShootTable::maybeGoalieCatch( const WorldModel & wm,
                               Shot * shot )
 {
     static const
-        Rect2D penalty_area( ServerParam::i().theirPenaltyAreaLine(), // left
+        Rect2D penalty_area( ServerParam::i().theirPenaltyAreaLineX(), // left
                              -ServerParam::i().penaltyAreaHalfWidth(), // top
                              ServerParam::i().penaltyAreaLength(), // length
                              ServerParam::i().penaltyAreaWidth() ); // width
@@ -319,23 +327,23 @@ ShootTable::maybeGoalieCatch( const WorldModel & wm,
     const double dash_accel_mag = ( ServerParam::i().maxPower()
                                     * ServerParam::i().defaultDashPowerRate()
                                     * ServerParam::i().defaultEffortMax() );
+    const double seen_dist_noise = goalie->distFromSelf() * 0.05;
 
     int min_cycle = 1;
     {
         Line2D shot_line( wm.ball().pos(), shot->point_ );
         double goalie_line_dist
             = std::max( 0.0, shot_line.dist( goalie->pos() ) - catchable_area );
+        goalie_line_dist -= seen_dist_noise;
         min_cycle = static_cast< int >
             ( std::ceil( goalie_line_dist / ServerParam::i().defaultRealSpeedMax() ) ) ;
         min_cycle -= std::min( 5, goalie->posCount() );
         min_cycle = std::max( 1, min_cycle );
     }
-    Vector2D ball_pos = wm.ball().pos() + shot->vel_;
-
-    ball_pos = inertia_n_step_point( wm.ball().pos(),
-                                     shot->vel_,
-                                     min_cycle,
-                                     ServerParam::i().ballDecay() );
+    Vector2D ball_pos = inertia_n_step_point( wm.ball().pos(),
+                                              shot->vel_,
+                                              min_cycle,
+                                              ServerParam::i().ballDecay() );
     Vector2D ball_vel = ( shot->vel_
                           * std::pow( ServerParam::i().ballDecay(), min_cycle ) );
 
@@ -350,7 +358,7 @@ ShootTable::maybeGoalieCatch( const WorldModel & wm,
                                     cycle,
                                     ServerParam::i().defaultPlayerDecay() );
         Vector2D ball_relative = ball_pos - goalie_pos;
-        double ball_dist = ball_relative.r();
+        double ball_dist = ball_relative.r() - seen_dist_noise;
 
         if ( ball_dist < catchable_area )
         {
@@ -436,7 +444,7 @@ ShootTable::maybeGoalieCatch( const WorldModel & wm,
             goalie_travel += goalie_vel.r();
             goalie_vel *= ServerParam::i().defaultPlayerDecay();
 
-            double d = goalie_pos.dist( ball_pos );
+            double d = goalie_pos.dist( ball_pos ) - seen_dist_noise;
 #if 0
             dlog.addText( Logger::SHOOT,
                           "_   cycle=%d ball(%.1f %.1f) angle=%.0f goalie pos(%.1f %.1f) travel=%.1f dist=%.1f turn=%d dash=%d",
@@ -456,7 +464,8 @@ ShootTable::maybeGoalieCatch( const WorldModel & wm,
 
         // check distance
         if ( goalie->pos().dist( goalie_pos ) * 1.05
-             > goalie->pos().dist( ball_pos ) - catchable_area - 0.2 )
+             > goalie->pos().dist( ball_pos ) - seen_dist_noise
+             - catchable_area - 0.2 )
         {
             return true;
         }
