@@ -38,6 +38,7 @@
 #include "global_visual_sensor.h"
 
 #include <rcsc/common/server_param.h>
+#include <rcsc/common/player_param.h>
 #include <rcsc/common/player_type.h>
 #include <rcsc/geom/rect_2d.h>
 
@@ -63,6 +64,9 @@ GlobalWorldModel::GlobalWorldModel()
         M_player_types_left[i] = Hetero_Default;
         M_player_types_right[i] = Hetero_Default;
     }
+
+    M_player_type_used_count_left.push_back( 11 );
+    M_player_type_used_count_right.push_back( 11 );
 }
 
 /*-------------------------------------------------------------------*/
@@ -78,6 +82,16 @@ GlobalWorldModel::init( const SideID side )
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+GlobalWorldModel::setAudioMemory( boost::shared_ptr< AudioMemory > memory )
+{
+    M_audio_memory = memory;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 GlobalWorldModel::initFreeformCount()
@@ -121,14 +135,14 @@ GlobalWorldModel::setTeamName( const SideID side,
 
 */
 void
-GlobalWorldModel::recvChangePlayerType( const SideID side,
-                                        const int unum,
-                                        const HeteroID type )
+GlobalWorldModel::setPlayerType( const SideID side,
+                                 const int unum,
+                                 const int type )
 {
     if ( side == NEUTRAL )
     {
         std::cerr << __FILE__ << ':' << __LINE__
-                  << " ***ERROR*** Invalid side = " << side
+                  << " ***ERROR*** Illegal side = " << side
                   << std::endl;
         return;
     }
@@ -136,7 +150,17 @@ GlobalWorldModel::recvChangePlayerType( const SideID side,
     if ( unum < 1 || 11 < unum )
     {
         std::cerr << __FILE__ << ':' << __LINE__
-                  << " ***ERROR*** Invalid unum " << unum
+                  << " ***ERROR*** Illegal unum " << unum
+                  << std::endl;
+        return;
+    }
+
+    if ( type != Hetero_Unknown
+         && ( type < Hetero_Default
+              || PlayerParam::i().playerTypes() <= type ) )
+    {
+        std::cerr << __FILE__ << ':' << __LINE__
+                  << " ***ERROR*** Illegal player type id " << type
                   << std::endl;
         return;
     }
@@ -144,18 +168,38 @@ GlobalWorldModel::recvChangePlayerType( const SideID side,
     if ( side == LEFT )
     {
         M_player_types_left[unum - 1] = type;
-        if ( M_game_mode.type() != GameMode::BeforeKickOff )
+
+        if ( this->time().cycle() > 0 )
         {
             ++M_change_count_left;
         }
+
+        M_player_type_used_count_left.assign( PlayerParam::i().playerTypes(), 0 );
+        for ( int i = 0; i < 11; ++i )
+        {
+            if ( M_player_types_left[i] != Hetero_Unknown )
+            {
+                M_player_type_used_count_left[ M_player_types_left[i] ] += 1;
+            }
+        }
     }
     else
     {
         M_player_types_right[unum - 1] = type;
-        if ( M_game_mode.type() != GameMode::BeforeKickOff )
+
+        if ( this->time().cycle() > 0 )
         {
             ++M_change_count_right;
         }
+
+        M_player_type_used_count_right.assign( PlayerParam::i().playerTypes(), 0 );
+        for ( int i = 0; i < 11; ++i )
+        {
+            if ( M_player_types_right[i] != Hetero_Unknown )
+            {
+                M_player_type_used_count_right[ M_player_types_right[i] ] += 1;
+            }
+        }
     }
 
     if ( side != ourSide()
@@ -236,7 +280,9 @@ GlobalWorldModel::updateAfterSeeGlobal( const GlobalVisualSensor & see_global,
 
     M_see_time = current;
 
+    //
     // update team names
+    //
 
     if ( ! see_global.teamNameLeft().empty() )
     {
@@ -248,11 +294,15 @@ GlobalWorldModel::updateAfterSeeGlobal( const GlobalVisualSensor & see_global,
         M_team_name_right = see_global.teamNameRight();
     }
 
+    //
     // update ball
+    //
 
     M_ball = see_global.ball();
 
+    //
     // update player instance variables
+    //
 
     std::list< GlobalPlayerObject > new_players;
 
@@ -286,8 +336,9 @@ GlobalWorldModel::updateAfterSeeGlobal( const GlobalVisualSensor & see_global,
     M_players.clear();
     M_players.splice( M_players.end(), new_players );
 
-
+    //
     // update the vector of player's pointer
+    //
 
     M_players_left.clear();
     M_players_right.clear();
@@ -307,14 +358,15 @@ GlobalWorldModel::updateAfterSeeGlobal( const GlobalVisualSensor & see_global,
         }
     }
 
-
-    // analyzer player type
+    //
+    // analyze player type
+    //
 
     M_player_type_analyzer.update();
 
     for ( int unum = 1; unum <= 11; ++unum )
     {
-        HeteroID id = M_player_type_analyzer.heteroID( unum );
+        int id = M_player_type_analyzer.heteroID( unum );
         if ( id != Hetero_Unknown )
         {
             if ( ourSide() == LEFT )
@@ -327,15 +379,38 @@ GlobalWorldModel::updateAfterSeeGlobal( const GlobalVisualSensor & see_global,
             }
         }
     }
+
+    if ( ourSide() == RIGHT )
+    {
+        M_player_type_used_count_left.assign( PlayerParam::i().playerTypes(), 0 );
+        for ( int i = 0; i < 11; ++i )
+        {
+            if ( M_player_types_left[i] != Hetero_Unknown )
+            {
+                M_player_type_used_count_left[ M_player_types_left[i] ] += 1;
+            }
+        }
+    }
+    else
+    {
+        M_player_type_used_count_right.assign( PlayerParam::i().playerTypes(), 0 );
+        for ( int i = 0; i < 11; ++i )
+        {
+            if ( M_player_types_right[i] != Hetero_Unknown )
+            {
+                M_player_type_used_count_right[ M_player_types_right[i] ] += 1;
+            }
+        }
+    }
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
 */
-HeteroID
-GlobalWorldModel::getHeteroID( const SideID side,
-                               const int unum ) const
+int
+GlobalWorldModel::heteroID( const SideID side,
+                            const int unum ) const
 {
     if ( side == NEUTRAL )
     {
@@ -353,14 +428,9 @@ GlobalWorldModel::getHeteroID( const SideID side,
         return Hetero_Unknown;
     }
 
-    if ( side == LEFT )
-    {
-        return M_player_types_left[unum-1];
-    }
-    else
-    {
-        return M_player_types_right[unum-1];
-    }
+    return ( side == LEFT
+             ? M_player_types_left[unum - 1]
+             : M_player_types_right[unum - 1] );
 }
 
 /*-------------------------------------------------------------------*/
@@ -457,7 +527,7 @@ GlobalWorldModel::existKickablePlayer() const
           it != end;
           ++it )
     {
-        HeteroID type = getHeteroID( it->side(), it->unum() );
+        int type = heteroID( it->side(), it->unum() );
         const PlayerType * param = PlayerTypeSet::i().get( type );
         double kickable_area = ( param
                                  ? param->kickableArea()
