@@ -34,6 +34,7 @@
 #endif
 
 #include "player_type.h"
+#include "player_param.h"
 #include "server_param.h"
 
 #include <rcsc/rcg/util.h>
@@ -63,7 +64,7 @@ PlayerType::PlayerType( const ServerParam & sparam,
                         const double & estamina,
                         const double & effmax,
                         const double & effmin )
-    : M_id( static_cast< HeteroID >( id ) )
+    : M_id( id )
     , M_player_speed_max( pspdmax )
     , M_stamina_inc_max( sincmax )
     , M_player_decay( pdecay )
@@ -77,8 +78,8 @@ PlayerType::PlayerType( const ServerParam & sparam,
     , M_effort_min( effmin )
     , M_cycles_to_reach_max_speed( 6 )
 {
-    if ( id < static_cast< int >( Hetero_Unknown )
-         || static_cast< int >( Hetero_MAX ) <= id )
+    if ( id < Hetero_Unknown
+         || PlayerParam::i().playerTypes() <= id )
     {
         std::cerr << "PlayerType. Invalid player type id " << id
                   << std::endl;
@@ -135,7 +136,7 @@ PlayerType::PlayerType( const ServerParam & sparam,
 PlayerType::PlayerType( const ServerParam & sparam,
                         const rcg::player_type_t & from )
 {
-    M_id = static_cast< HeteroID >( rcg::nstohi( from.id ) );
+    M_id = rcg::nstohi( from.id );
     M_player_speed_max = rcg::nltohd( from.player_speed_max );
     M_stamina_inc_max = rcg::nltohd( from.stamina_inc_max );
     M_player_decay = rcg::nltohd( from.player_decay );
@@ -171,7 +172,7 @@ PlayerType::PlayerType( const ServerParam & sparam,
 void
 PlayerType::convertTo( rcg::player_type_t & to ) const
 {
-    to.id = rcg::hitons( static_cast< int >( M_id ) );
+    to.id = rcg::hitons( M_id );
     to.player_speed_max = rcg::hdtonl( M_player_speed_max );
     to.stamina_inc_max = rcg::hdtonl( M_stamina_inc_max );
     to.player_decay = rcg::hdtonl( M_player_decay );
@@ -201,6 +202,33 @@ PlayerType::convertTo( rcg::player_type_t & to ) const
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+std::string
+PlayerType::toStr() const
+{
+    std::ostringstream os;
+
+    os << "(player_type "
+       << "(id " << M_id << ')'
+       << "(player_speed_max " << M_player_speed_max << ')'
+       << "(stamina_inc_max " << M_stamina_inc_max << ')'
+       << "(player_decay " << M_player_decay << ')'
+       << "(inertia_moment " << M_inertia_moment << ')'
+       << "(dash_power_rate " << M_dash_power_rate << ')'
+       << "(player_size " << M_player_size << ')'
+       << "(kickable_margin " << M_kickable_margin << ')'
+       << "(kick_rand " << M_kick_rand << ')'
+       << "(extra_stamina " << M_extra_stamina << ')'
+       << "(effort_max " << M_effort_max << ')'
+       << "(effort_min " << M_effort_min << ')'
+       << ')';
+
+    return os.str();
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 PlayerType::parseV8( const char * msg )
@@ -219,76 +247,81 @@ PlayerType::parseV8( const char * msg )
       (extra_stamina 98) (effort_max 0.804) (effort_min 0.404))";
     */
 
-    // skip param specifier "(player_type"
-    while ( *msg != '\0' && *msg != ' ' ) ++msg;
-    ++msg; // skip space
-
+    int n_read = 0;
 
     char name[32];
     int id = 0;
-    if ( std::sscanf( msg, "(%s %d)", name, &id ) < 2 )
+    if ( std::sscanf( msg, " ( player_type ( %s %d ) %n ",
+                      name, &id, &n_read ) != 2
+         || n_read == 0
+         || id < 0 )
     {
-        std::cerr << "id read error" << std::endl;
+        std::cerr << "PlayerType id read error. "
+                  << msg
+                  << std::endl;
         return;
     }
-    M_id = static_cast< HeteroID >( id );
+    msg += n_read;
 
-    while ( *msg != ')' ) msg++;
-    while ( *msg != '\0' && *msg != '(' ) msg++;
+    M_id = id;
 
-    double tmpval;
     int n_param = 0;
     while ( *msg != '\0' && *msg != ')' )
     {
-        if ( std::sscanf( msg, "(%s %lf)", name, &tmpval ) != 2 )
+        double val = 0.0;
+        if ( std::sscanf( msg, " ( %s %lf ) %n ",
+                          name, &val, &n_read ) != 2
+             || n_read == 0 )
         {
-            std::cerr << "id read error: " << msg << std::endl;
+            std::cerr << "PlayerType parameter read error: "
+                      << msg << std::endl;
             break;
         }
+        msg += n_read;
 
         if ( ! std::strcmp( name, "player_speed_max" ) )
         {
-            M_player_speed_max = tmpval;
+            M_player_speed_max = val;
         }
         else if ( ! std::strcmp( name, "stamina_inc_max" ) )
         {
-            M_stamina_inc_max = tmpval;
+            M_stamina_inc_max = val;
         }
         else if ( ! std::strcmp( name, "player_decay" ) )
         {
-            M_player_decay = tmpval;
+            M_player_decay = val;
         }
         else if ( ! std::strcmp( name, "inertia_moment" ) )
         {
-            M_inertia_moment = tmpval;
+            M_inertia_moment = val;
         }
         else if ( ! std::strcmp( name, "dash_power_rate" ) )
         {
-            M_dash_power_rate = tmpval;
+            M_dash_power_rate = val;
         }
         else if ( ! std::strcmp( name, "player_size" ) )
         {
-            M_player_size = tmpval;
+            M_player_size = val;
         }
         else if ( ! std::strcmp( name, "kickable_margin" ) )
         {
-            M_kickable_margin = tmpval;
+            M_kickable_margin = val;
         }
         else if ( ! std::strcmp( name, "kick_rand" ) )
         {
-            M_kick_rand = tmpval;
+            M_kick_rand = val;
         }
         else if ( ! std::strcmp( name, "extra_stamina" ) )
         {
-            M_extra_stamina = tmpval;
+            M_extra_stamina = val;
         }
         else if ( ! std::strcmp( name, "effort_max" ) )
         {
-            M_effort_max = tmpval;
+            M_effort_max = val;
         }
         else if ( ! std::strcmp( name, "effort_min" ) )
         {
-            M_effort_min = tmpval;
+            M_effort_min = val;
         }
         else
         {
@@ -297,8 +330,6 @@ PlayerType::parseV8( const char * msg )
         }
 
         ++n_param;
-        while ( *msg != ')' ) ++msg;
-        while ( *msg != '\0' && *msg != '(' ) ++msg;
     }
 
     //std::cerr << "  read " << n_param << " params" << endl;
@@ -337,9 +368,8 @@ PlayerType::parseV7( const char * msg )
     std::istringstream msg_strm( msg );
     std::string tmp;
 
-    int id;
     msg_strm >> tmp // skip "(player_type"
-             >> id
+             >> M_id
              >> M_player_speed_max
              >> M_stamina_inc_max
              >> M_player_decay
@@ -351,7 +381,6 @@ PlayerType::parseV7( const char * msg )
              >> M_extra_stamina
              >> M_effort_max
              >> M_effort_min;
-    M_id = static_cast< HeteroID >( id );
 }
 
 /*-------------------------------------------------------------------*/
@@ -781,7 +810,7 @@ PlayerType::predictStaminaAfterNrDash( const ServerParam & sparam,
 std::ostream &
 PlayerType::print( std::ostream & os ) const
 {
-    os << "player_type id : " << static_cast< int >( id() )
+    os << "player_type id : " << id()
        << "\n  player_speed_max : " << playerSpeedMax()
        << "\n  stamina_inc_max :  " << staminaIncMax()
        << "\n  player_decay : "     << playerDecay()
@@ -831,7 +860,7 @@ PlayerTypeSet::PlayerTypeSet()
                     1.0, // effort max
                     0.6 ) // effort min
 {
-
+    resetDefaultType( ServerParam::i() );
 }
 
 /*-------------------------------------------------------------------*/
@@ -862,7 +891,8 @@ PlayerTypeSet::resetDefaultType( const ServerParam & sparam )
 void
 PlayerTypeSet::insert( const PlayerType & param )
 {
-    if ( M_player_type_map.size() >= Hetero_MAX )
+    if ( static_cast< int >( M_player_type_map.size() )
+         >= PlayerParam::i().playerTypes() )
     {
         std::cerr << "player types over flow" << std::endl;
         return;
@@ -872,8 +902,8 @@ PlayerTypeSet::insert( const PlayerType & param )
     M_player_type_map.insert( std::make_pair( param.id(), param ) );
 
 
-    if ( M_player_type_map.size()
-         == static_cast< std::size_t >( Hetero_MAX ) )
+    if ( static_cast< int >( M_player_type_map.size() )
+         == PlayerParam::i().playerTypes() )
     {
         createDummyType();
     }
@@ -952,7 +982,7 @@ PlayerTypeSet::createDummyType()
 */
 const
 PlayerType *
-PlayerTypeSet::get( const HeteroID id ) const
+PlayerTypeSet::get( const int id ) const
 {
     if ( id == Hetero_Unknown )
     {
