@@ -75,7 +75,7 @@ const double ServerParam::DEFAULT_PLAYER_SIZE = 0.3;
 const double ServerParam::DEFAULT_PLAYER_DECAY = 0.4;
 const double ServerParam::DEFAULT_PLAYER_RAND = 0.05;
 const double ServerParam::DEFAULT_PLAYER_WEIGHT = 60.0;
-const double ServerParam::DEFAULT_PLAYER_SPEED_MAX =1.2;
+const double ServerParam::DEFAULT_PLAYER_SPEED_MAX = 1.2;
 const double ServerParam::DEFAULT_PLAYER_ACCEL_MAX = 1.0;
 
 const double ServerParam::DEFAULT_STAMINA_MAX = 4000.0;
@@ -308,6 +308,11 @@ const std::string ServerParam::DEFAULT_MODULE_DIR = "";
 
 const double ServerParam::DEFAULT_BALL_STUCK_AREA = 3.0;
 
+const double ServerParam::DEFAULT_MAX_TACKLE_POWER = 100.0;
+const double ServerParam::DEFAULT_MAX_BACK_TACKLE_POWER = 50.0;
+const double ServerParam::DEFAULT_PLAYER_SPEED_MAX_MIN = 0.8;
+const double ServerParam::DEFAULT_EXTRA_STAMINA = 0.0;
+
 /*
   It is necessary to check out whether server is NEW_QSTEP mode
   bool defined_qstep_l = false;
@@ -334,7 +339,7 @@ ServerParam::instance()
 
 */
 ServerParam::ServerParam()
-    : M_param_map( new ParamMap )
+    : M_param_map( new ParamMap( "server_param" ) )
 {
     assert( M_param_map );
 
@@ -579,7 +584,18 @@ ServerParam::setDefaultParam()
 
     M_module_dir = DEFAULT_MODULE_DIR;
 
+    // 11.0.0
     M_ball_stuck_area = DEFAULT_BALL_STUCK_AREA;
+    // M_coach_msg_file = "";
+
+    // 12.0.0
+    M_max_tackle_power = DEFAULT_MAX_TACKLE_POWER;
+    M_max_back_tackle_power = DEFAULT_MAX_BACK_TACKLE_POWER;
+    M_player_speed_max_min = DEFAULT_PLAYER_SPEED_MAX_MIN;
+    M_extra_stamina = DEFAULT_EXTRA_STAMINA;
+    M_synch_see_offset = 30;
+
+    M_max_monitors = -1;
 
     setAdditionalParam();
 }
@@ -591,9 +607,10 @@ ServerParam::setDefaultParam()
 void
 ServerParam::setAdditionalParam()
 {
-    // catchable area
+    M_kickable_area = M_kickable_margin + M_ball_size + M_player_size;
     M_catchable_area = std::sqrt( std::pow( catchAreaWidth() * 0.5, 2.0 )
                                   + std::pow( catchAreaLength(), 2.0 ) );
+    M_control_radius_width = M_control_radius - M_player_size;
 
     // real speed max
     double accel_max = maxPower() * defaultDashPowerRate() * defaultEffortMax();
@@ -655,7 +672,7 @@ ServerParam::createMap()
         ( "kick_power_rate", "", &M_kick_power_rate )
         ( "kickable_margin", "", &M_kickable_margin )
         ( "control_radius", "", &M_control_radius )
-        ( "control_radius_width", "", &M_control_radius_width ) // not needed
+        //( "control_radius_width", "", &M_control_radius_width ) // not needed
 
         ( "maxpower", "", &M_max_power )
         ( "minpower", "", &M_min_power )
@@ -674,7 +691,7 @@ ServerParam::createMap()
         ( "wind_ang", "", &M_wind_angle )
         ( "wind_rand", "", &M_wind_rand )
 
-        ( "kickable_area", "", &M_kickable_area ) // not needed
+        //( "kickable_area", "", &M_kickable_area ) // not needed
 
         ( "catchable_area_l", "", &M_catch_area_l )
         ( "catchable_area_w", "", &M_catch_area_w )
@@ -703,7 +720,7 @@ ServerParam::createMap()
         ( "send_step", "", &M_send_step )
         ( "recv_step", "", &M_recv_step )
         ( "sense_body_step", "", &M_sense_body_step )
-        ( "lcm_step", "", &M_lcm_step )
+        //( "lcm_step", "", &M_lcm_step ) // not needed
 
         ( "say_msg_size", "", &M_player_say_msg_size )
         ( "hear_max", "", &M_player_hear_max )
@@ -722,13 +739,13 @@ ServerParam::createMap()
 
         ( "quantize_step", "", &M_dist_quantize_step )
         ( "quantize_step_l", "", &M_landmark_dist_quantize_step )
-        ( "quantize_step_dir", "", &M_dir_quantize_step )
-        ( "quantize_step_dist_team_l", "", &M_dist_quantize_step_l )
-        ( "quantize_step_dist_team_r", "", &M_dist_quantize_step_r )
-        ( "quantize_step_dist_l_team_l", "", &M_landmark_dist_quantize_step_l )
-        ( "quantize_step_dist_l_team_r", "", &M_landmark_dist_quantize_step_r )
-        ( "quantize_step_dir_team_l", "", &M_dir_quantize_step_l )
-        ( "quantize_step_dir_team_r", "", &M_dir_quantize_step_r )
+        //( "quantize_step_dir", "", &M_dir_quantize_step )
+        //( "quantize_step_dist_team_l", "", &M_dist_quantize_step_l )
+        //( "quantize_step_dist_team_r", "", &M_dist_quantize_step_r )
+        //( "quantize_step_dist_l_team_l", "", &M_landmark_dist_quantize_step_l )
+        //( "quantize_step_dist_l_team_r", "", &M_landmark_dist_quantize_step_r )
+        //( "quantize_step_dir_team_l", "", &M_dir_quantize_step_l )
+        //( "quantize_step_dir_team_r", "", &M_dir_quantize_step_r )
 
         ( "coach", "", BoolSwitch( &M_coach_mode ) )
         ( "coach_w_referee", "", BoolSwitch( &M_coach_with_referee_mode ) )
@@ -849,6 +866,16 @@ ServerParam::createMap()
 
         ( "ball_stuck_area", "", &M_ball_stuck_area )
         ( "coach_msg_file", "", &M_coach_msg_file )
+
+        ( "max_tackle_power", "", &M_max_tackle_power )
+        ( "max_back_tackle_power", "", &M_max_back_tackle_power )
+        ( "player_speed_max_min", "", &M_player_speed_max_min )
+        ( "extra_stamina", "", &M_extra_stamina )
+        ( "synch_see_offset", "", &M_synch_see_offset )
+
+        ( "max_monitors", "", &M_max_monitors )
+        ( "min_catch_probability", "", &M_min_catch_probability )
+        ( "reliable_catch_area_l", "", &M_reliable_catch_area_l )
         ;
 }
 
@@ -875,9 +902,6 @@ ServerParam::parse( const char * msg,
         result = parseV7( msg );
     }
 
-    M_kickable_area = M_kickable_margin + M_ball_size + M_player_size;
-    M_control_radius_width = M_control_radius - M_player_size;
-
     setAdditionalParam();
 
     return result;
@@ -1085,6 +1109,8 @@ ServerParam::parseV7( const char * msg )
 void
 ServerParam::convertFrom( const rcg::server_params_t & from )
 {
+    double tmp = 0.0;
+
     M_goal_width = rcg::nltohd( from.goal_width );
     M_inertia_moment = rcg::nltohd( from.inertia_moment );
 
@@ -1215,15 +1241,24 @@ ServerParam::convertFrom( const rcg::server_params_t & from )
     M_keepaway_length = rcg::nltohd( from.ka_length );
     M_keepaway_width = rcg::nltohd( from.ka_width );
 
-    M_ball_stuck_area = rcg::nltohd( from.ball_stuck_area );
-    /*
-      from.sparelong6;
-      from.sparelong7;
-      from.sparelong8;
-      from.sparelong9;
-      from.sparelong10;
-    */
+    // 11.0.0
+    tmp = rcg::nltohd( from.ball_stuck_area );
+    if ( std::fabs( tmp ) < 1000.0 ) M_ball_stuck_area = tmp;
 
+    // 12.0.0
+    tmp = rcg::nltohd( from.max_tackle_power );
+    if ( 0.0 < tmp && std::fabs( tmp ) < 1000.0 ) M_max_tackle_power = tmp;
+    tmp = rcg::nltohd( from.max_back_tackle_power );
+    if ( 0.0 < tmp && std::fabs( tmp ) < 1000.0 ) M_max_back_tackle_power = tmp;
+
+    tmp = rcg::nltohd( from.tackle_dist );
+    if ( 0.0 <= tmp && std::fabs( tmp ) < 1000.0 ) M_tackle_dist = tmp;
+    tmp = rcg::nltohd( from.tackle_back_dist );
+    if ( 0.0 <= tmp && std::fabs( tmp ) < 1000.0 ) M_tackle_back_dist = tmp;
+    tmp = rcg::nltohd( from.tackle_width );
+    if ( 0.0 < tmp && std::fabs( tmp ) < 1000.0 ) M_tackle_width = tmp;
+
+    //
     M_start_goal_l = rcg::nstohi( from.start_goal_l );
     M_start_goal_r = rcg::nstohi( from.start_goal_r );
 
@@ -1275,7 +1310,7 @@ ServerParam::convertTo( rcg::server_params_t & to ) const
     to.effort_inc = rcg::hdtonl( M_effort_inc );
 
     to.kick_rand = rcg::hdtonl( M_kick_rand );
-    to.team_actuator_noise = rcg::hdtonl( M_team_actuator_noise );
+    to.team_actuator_noise = rcg::hbtons( M_team_actuator_noise );
     to.player_rand_factor_l = rcg::hdtonl( M_player_rand_factor_l );
     to.player_rand_factor_r = rcg::hdtonl( M_player_rand_factor_r );
     to.kick_rand_factor_l = rcg::hdtonl( M_kick_rand_factor_l );
@@ -1380,14 +1415,15 @@ ServerParam::convertTo( rcg::server_params_t & to ) const
     to.ka_length = rcg::hdtonl( M_keepaway_length );
     to.ka_width = rcg::hdtonl( M_keepaway_width );
 
+    // 11.0.0
     to.ball_stuck_area = rcg::hdtonl( M_ball_stuck_area );
-    /*
-      from.sparelong6;
-      from.sparelong7;
-      from.sparelong8;
-      from.sparelong9;
-      from.sparelong10;
-    */
+    // 12.0.0
+    to.max_tackle_power = rcg::hdtonl( M_max_tackle_power );
+    to.max_back_tackle_power = rcg::hdtonl( M_max_back_tackle_power );
+
+    to.tackle_dist = rcg::hdtonl( M_tackle_dist );
+    to.tackle_back_dist = rcg::hdtonl( M_tackle_back_dist );
+    to.tackle_width = rcg::hdtonl( M_tackle_width );
 
     to.start_goal_l = rcg::hitons( M_start_goal_l );
     to.start_goal_r = rcg::hitons( M_start_goal_r );
@@ -1405,4 +1441,39 @@ ServerParam::convertTo( rcg::server_params_t & to ) const
     to.point_to_duration = rcg::hitons( M_point_to_duration );
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+std::string
+ServerParam::toStr() const
+{
+    std::ostringstream os;
+
+    os << "(server_param ";
+
+    const std::map< std::string, ParamPtr >::const_iterator end = M_param_map->longNameMap().end();
+    for ( std::map< std::string, ParamPtr >::const_iterator it = M_param_map->longNameMap().begin();
+          it != end;
+          ++it )
+    {
+        os << '(' << it->second->longName() << ' ';
+        if ( it->second->isString() )
+        {
+            os << '"';
+            it->second->printValue( os );
+            os << '"';
+        }
+        else
+        {
+            it->second->printValue( os );
+        }
+        os << ')';
+    }
+
+    os << ')';
+
+    return os.str();
+}
+
 }
