@@ -32,6 +32,8 @@
 #ifndef RCSC_PARAM_SERVER_PARAM_H
 #define RCSC_PARAM_SERVER_PARAM_H
 
+#include <rcsc/geom/vector_2d.h>
+
 #include <boost/scoped_ptr.hpp>
 
 #include <string>
@@ -86,6 +88,7 @@ public:
     static const double DEFAULT_PLAYER_RAND;
     static const double DEFAULT_PLAYER_WEIGHT;
     static const double DEFAULT_PLAYER_SPEED_MAX;
+    static const double DEFAULT_PLAYER_SPEED_MAX_MIN;
     static const double DEFAULT_PLAYER_ACCEL_MAX;
 
     static const double DEFAULT_STAMINA_MAX;
@@ -312,6 +315,11 @@ public:
 
     static const double DEFAULT_BALL_STUCK_AREA;
 
+    static const bool DEFAULT_ALLOW_MULT_DEFAULT_TYPE;
+    static const double DEFAULT_MAX_TACKLE_POWER;
+    static const double DEFAULT_MAX_BACK_TACKLE_POWER;
+    static const double DEFAULT_EXTRA_STAMINA;
+
 private:
 
     //////////////////////////////////////////////////////
@@ -566,6 +574,20 @@ private:
     double M_ball_stuck_area;
     std::string M_coach_msg_file;
 
+    // rcssserver-12.0.0
+    double M_max_tackle_power;
+    double M_max_back_tackle_power;
+    double M_player_speed_max_min;
+    double M_extra_stamina;
+    int M_synch_see_offset;
+
+    // 999
+    int M_max_monitors;
+
+    // test
+    double M_min_catch_probability;
+    double M_reliable_catch_area_l;
+
     //--------------------------------------------------------
     // additional params
     double M_catchable_area; //!< real catchable length (diagonal line length)
@@ -654,7 +676,14 @@ public:
      */
     void convertTo( rcg::server_params_t & to ) const;
 
-    // fixed parameters' value
+    /*!
+      \brief convert to the rcss parameter message
+      \return parameter message string
+     */
+    std::string toStr() const;
+
+
+    // static parameters
 
     const
     int maxPlayer() const
@@ -855,11 +884,6 @@ public:
           return M_effort_inc;
       }
 
-    double defaultExtraStamina() const
-      {
-          return 0.0;
-      }
-
     // hetero param
     const
     double & defaultKickRand() const
@@ -1696,6 +1720,8 @@ public:
           return M_module_dir;
       }
 
+    // 11.0.0
+
     const
     double & ballStuckArea() const
       {
@@ -1707,76 +1733,124 @@ public:
           return M_coach_msg_file;
       }
 
+    // 12.0.0
+
+    const
+    double & maxTacklePower() const
+      {
+          return M_max_tackle_power;
+      }
+    const
+    double & maxBackTacklePower() const
+      {
+          return M_max_back_tackle_power;
+      }
+    const
+    double & playerSpeedMaxMin() const
+      {
+          return M_player_speed_max_min;
+      }
+    const
+    double & defaultExtraStamina() const
+      {
+          return M_extra_stamina;
+      }
+
+    int synchSeeOffset() const
+      {
+          return M_synch_see_offset;
+      }
+
+    int maxMonitors() const
+      {
+          return M_max_monitors;
+      }
+
 
     // automatically defined values
 
-    const
     double pitchHalfLength() const
       {
           return pitchLength() * 0.5;
       }
-    const
+
     double pitchHalfWidth() const
       {
           return pitchWidth() * 0.5;
       }
-    const
+
     double penaltyAreaHalfWidth() const
       {
           return penaltyAreaWidth() * 0.5;
       }
-    const
+
     double goalAreaHalfWidth() const
       {
           return goalAreaWidth() * 0.5;
       }
-    const
+
     double goalHalfWidth() const
       {
           return goalWidth() * 0.5;
       }
 
-    const
-    double ourPenaltyAreaLine() const
+    double ourTeamGoalLineX() const
+      {
+          return - pitchHalfLength();
+      }
+
+    double theirTeamGoalLineX() const
+      {
+          return + pitchHalfLength();
+      }
+
+    Vector2D ourTeamGoalPos() const
+      {
+          return Vector2D( - pitchHalfLength(), 0.0 );
+      }
+
+    Vector2D theirTeamGoalPos() const
+      {
+          return Vector2D( + pitchHalfLength(), 0.0 );
+      }
+
+    double ourPenaltyAreaLineX() const
       {
           return -pitchHalfLength() + penaltyAreaLength();
       }
-    const
-    double theirPenaltyAreaLine() const
+
+    double theirPenaltyAreaLineX() const
       {
           return pitchHalfLength() - penaltyAreaLength();
       }
 
-    const
-    double recoverDecThrValue() const
-      {
-          return recoverDecThr() * staminaMax();
-      }
-    const
-    double effortDecThrValue() const
-      {
-          return effortDecThr() * staminaMax();
-      }
-    const
-    double effortIncThrValue() const
-      {
-          return effortIncThr() * staminaMax();
-      }
-
-
-    // additional params
-    const
-    double & catchableArea() const
-      {
-          return M_catchable_area;
-      }
-
     const
     double & defaultRealSpeedMax() const
       {
           return M_real_speed_max;
       }
 
+    double recoverDecThrValue() const
+      {
+          return recoverDecThr() * staminaMax();
+      }
+
+    double effortDecThrValue() const
+      {
+          return effortDecThr() * staminaMax();
+      }
+
+    double effortIncThrValue() const
+      {
+          return effortIncThr() * staminaMax();
+      }
+
+    // additional params
+    const
+    double & catchableArea() const
+      {
+          return M_catchable_area;
+      }
 
     // utility
 
