@@ -44,7 +44,7 @@ namespace rcsc {
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 FormationDT::Param::Param( const Snapshot & data )
     : ball_( data.ball_ )
     , players_( data.players_ )
@@ -55,7 +55,7 @@ FormationDT::Param::Param( const Snapshot & data )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 Vector2D
 FormationDT::Param::getPosition( const int unum ) const
 {
@@ -77,7 +77,7 @@ FormationDT::Param::getPosition( const int unum ) const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 FormationDT::FormationDT()
     : Formation()
 {
@@ -92,7 +92,7 @@ FormationDT::FormationDT()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 Formation::Snapshot
 FormationDT::createDefaultParam()
 {
@@ -112,14 +112,14 @@ FormationDT::createDefaultParam()
     // 11(10): right forward
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "CenterBack", Formation::SIDE );
-    setMirrorType( 3, 2 );
+    setSynmetryType( 3, 2 );
     createNewRole( 4, "SideBack", Formation::SIDE );
-    setMirrorType( 5, 4 );
+    setSynmetryType( 5, 4 );
     createNewRole( 6, "DefensiveHalf", Formation::CENTER );
     createNewRole( 7, "OffensiveHalf", Formation::SIDE );
-    setMirrorType( 8, 7 );
+    setSynmetryType( 8, 7 );
     createNewRole( 9, "SideForward", Formation::SIDE );
-    setMirrorType( 10, 9 );
+    setSynmetryType( 10, 9 );
     createNewRole( 11, "CenterForward", Formation::CENTER );
 
     Snapshot snap;
@@ -154,14 +154,14 @@ FormationDT::createDefaultParam()
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "Sweeper", Formation::CENTER );
     createNewRole( 3, "SideBack", Formation::SIDE );
-    setMirrorType( 4, 3 );
+    setSynmetryType( 4, 3 );
     createNewRole( 5, "DefensiveHalf", Formation::CENTER );
     createNewRole( 6, "OffensiveHalf", Formation::SIDE );
-    setMirrorType( 7, 6 );
+    setSynmetryType( 7, 6 );
     createNewRole( 8, "SideHalf", Formation::SIDE );
-    setMirrorType( 9, 8 );
+    setSynmetryType( 9, 8 );
     createNewRole( 10, "Forward", Formation::SIDE );
-    setMirrorType( 11, 10 );
+    setSynmetryType( 11, 10 );
 
     Snapshot snap;
 
@@ -194,14 +194,14 @@ FormationDT::createDefaultParam()
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "Sweeper", Formation::CENTER );
     createNewRole( 3, "SideBack", Formation::SIDE );
-    setMirrorType( 4, 3 );
+    setSynmetryType( 4, 3 );
     createNewRole( 5, "DefensiveHalf", Formation::SIDE );
-    setMirrorType( 6, 5 );
+    setSynmetryType( 6, 5 );
     createNewRole( 7, "OffensiveHalf", Formation::CENTER );
     createNewRole( 8, "SideHalf", Formation::SIDE );
-    setMirrorType( 9, 8 );
+    setSynmetryType( 9, 8 );
     createNewRole( 10, "Forward", Formation::SIDE );
-    setMirrorType( 11, 10 );
+    setSynmetryType( 11, 10 );
 
     Snapshot snap;
 
@@ -224,7 +224,7 @@ FormationDT::createDefaultParam()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 FormationDT::setRoleName( const int unum,
                           const std::string & name )
@@ -243,7 +243,7 @@ FormationDT::setRoleName( const int unum,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 std::string
 FormationDT::getRoleName( const int unum ) const
 {
@@ -261,7 +261,7 @@ FormationDT::getRoleName( const int unum ) const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 FormationDT::createNewRole( const int unum,
                             const std::string & role_name,
@@ -284,7 +284,7 @@ FormationDT::createNewRole( const int unum,
     case Formation::SIDE:
         setSideType( unum );
         break;
-    case Formation::MIRROR:
+    case Formation::SYNMETRY:
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** Invalid side type "
                   << std::endl;
@@ -297,7 +297,7 @@ FormationDT::createNewRole( const int unum,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 Vector2D
 FormationDT::getPosition( const int unum,
                           const Vector2D & focus_point ) const
@@ -320,7 +320,7 @@ FormationDT::getPosition( const int unum,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 FormationDT::getPositions( const Vector2D & focus_point,
                            std::vector< Vector2D > & positions ) const
@@ -340,7 +340,7 @@ FormationDT::getPositions( const Vector2D & focus_point,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 Vector2D
 FormationDT::interpolate( const int unum,
                           const Vector2D & focus_point,
@@ -440,7 +440,7 @@ FormationDT::interpolate( const int unum,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 FormationDT::train( const std::list< Snapshot > & train_data )
 {
@@ -466,7 +466,7 @@ FormationDT::train( const std::list< Snapshot > & train_data )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 FormationDT::read( std::istream & is )
 {
@@ -481,8 +481,6 @@ FormationDT::read( std::istream & is )
         return false;
     }
 
-    std::string line_buf;
-
     // read role assignment
     if ( ! readRoles( is ) )
     {
@@ -494,7 +492,7 @@ FormationDT::read( std::istream & is )
     for ( int unum = 1; unum <= 11; ++unum )
     {
         std::cerr << "Role=" << getRoleName( unum )
-                  << " Mirror=" << getMirrorNumber( unum )
+                  << " Synmetry=" << getSynmetryNumber( unum )
                   << std::endl;
     }
 #endif
@@ -534,7 +532,7 @@ FormationDT::read( std::istream & is )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 FormationDT::readRoles( std::istream & is )
 {
@@ -553,11 +551,11 @@ FormationDT::readRoles( std::istream & is )
     for ( int unum = 1; unum <= 11; ++unum )
     {
         char role_name[128];
-        int mirror_number;
+        int synmetry_number;
         int n_read = 0;
 
         if ( std::sscanf( msg, " %s %d %n ",
-                          role_name, &mirror_number, &n_read ) != 2 )
+                          role_name, &synmetry_number, &n_read ) != 2 )
         {
             std::cerr << __FILE__ << ":" << __LINE__
                       << " *** ERROR *** readRoles(). Failed to read player "
@@ -567,11 +565,11 @@ FormationDT::readRoles( std::istream & is )
         }
         msg += n_read;
 
-        const Formation::SideType type = ( mirror_number == 0
+        const Formation::SideType type = ( synmetry_number == 0
                                            ? Formation::CENTER
-                                           : mirror_number < 0
+                                           : synmetry_number < 0
                                            ? Formation::SIDE
-                                           : Formation::MIRROR );
+                                           : Formation::SYNMETRY );
         if ( type == Formation::CENTER )
         {
             createNewRole( unum, role_name, type );
@@ -582,7 +580,7 @@ FormationDT::readRoles( std::istream & is )
         }
         else
         {
-            setMirrorType( unum, mirror_number );
+            setSynmetryType( unum, synmetry_number );
         }
     }
 
@@ -592,7 +590,7 @@ FormationDT::readRoles( std::istream & is )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 FormationDT::readSamples( std::istream & is )
 {
@@ -653,7 +651,7 @@ FormationDT::readSamples( std::istream & is )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 std::ostream &
 FormationDT::print( std::ostream & os ) const
 {
@@ -663,7 +661,7 @@ FormationDT::print( std::ostream & os ) const
     for ( int unum = 1; unum <= 11; ++unum )
     {
         os << M_role_name[unum - 1] << ' '
-           << M_mirror_number[unum - 1] << ' ';
+           << M_synmetry_number[unum - 1] << ' ';
     }
     os << '\n';
 
