@@ -39,23 +39,27 @@
 namespace rcsc {
 
 /*!
-  \class FormationTriangulation
-  \brief formation definition using Delaunay Triangulation
+  \class FormationDT
+  \brief formation which utilizes Delaunay Triangulation
 */
 class FormationDT
     : public Formation {
 public:
 
+    /*!
+      \struct Param
+      \brief training data object
+     */
     struct Param {
-        Vector2D M_ball; //!< ball position = focus point
-        std::vector< Vector2D > M_players; //!< all players' desired position
+        Vector2D ball_; //!< ball position = focus point
+        std::vector< Vector2D > players_; //!< all players' desired position
 
         /*!
           \brief just allocate memory
         */
         Param()
-            : M_ball( 0.0, 0.0 )
-            , M_players( 11 )
+            : ball_( 0.0, 0.0 )
+            , players_( 11 )
           { }
 
         /*!
@@ -70,8 +74,8 @@ public:
         */
         Param( const Vector2D & ball,
                const std::vector< Vector2D > & players )
-            : M_ball( ball )
-            , M_players( players )
+            : ball_( ball )
+            , players_( players )
           { }
 
         /*!
@@ -84,8 +88,8 @@ public:
         Param & assign( const Vector2D & ball,
                         const std::vector< Vector2D > & players )
           {
-              M_ball = ball;
-              M_players = players;
+              ball_ = ball;
+              players_ = players;
               return *this;
           }
 
@@ -115,6 +119,16 @@ public:
     */
     FormationDT();
 
+    /*!
+      \brief simple accessor.
+      \return const reference to the param instance
+     */
+    const
+    std::vector< Param > & params() const
+      {
+          return M_param;
+      }
+
     /*!
       \brief static method. get formation method name
       \return method name string
@@ -204,6 +218,15 @@ public:
     Vector2D getPosition( const int unum,
                           const Vector2D & focus_point ) const;
 
+    /*
+      \brief get all positions for the current focus point
+      \param focus_point current focus point, usually ball position
+      \param positions contaner to store the result
+     */
+    virtual
+    void getPositions( const Vector2D & focus_point,
+                       std::vector< Vector2D > & positions ) const;
+
     /*!
       \brief update formation paramter using training data set
       \param train_data training data container
@@ -229,6 +252,10 @@ public:
 
 private:
 
+    Vector2D interpolate( const int unum,
+                          const Vector2D & focus_point,
+                          const DelaunayTriangulation::TrianglePtr tri ) const;
+
     /*!
       \brief restore role assignment from the input stream
       \param is reference to the input stream
