@@ -50,7 +50,7 @@ const double FormationRBF::Param::PITCH_WIDTH = 68.0 + 10.0;
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 FormationRBF::Param::Param()
     : M_net( 2, 2 )
 {
@@ -60,7 +60,7 @@ FormationRBF::Param::Param()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 Vector2D
 FormationRBF::Param::getPosition( const Vector2D & ball_pos,
                                   const Formation::SideType type ) const
@@ -123,7 +123,7 @@ FormationRBF::Param::readRoleName( std::istream & is )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 FormationRBF::Param::readParam( std::istream & is )
 {
@@ -155,7 +155,7 @@ FormationRBF::Param::readParam( std::istream & is )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 FormationRBF::Param::read( std::istream & is )
 {
@@ -180,7 +180,7 @@ FormationRBF::Param::read( std::istream & is )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 std::ostream &
 FormationRBF::Param::printRoleName( std::ostream & os ) const
 {
@@ -198,7 +198,7 @@ FormationRBF::Param::printRoleName( std::ostream & os ) const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 std::ostream &
 FormationRBF::Param::printParam( std::ostream & os ) const
 {
@@ -231,7 +231,7 @@ FormationRBF::Param::print( std::ostream & os ) const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 FormationRBF::FormationRBF()
     : Formation()
 {
@@ -241,7 +241,7 @@ FormationRBF::FormationRBF()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 Formation::Snapshot
 FormationRBF::createDefaultParam()
 {
@@ -259,18 +259,18 @@ FormationRBF::createDefaultParam()
     // 11(10): right forward
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "CenterBack", Formation::SIDE );
-    createNewRole( 3, "CenterBack", Formation::MIRROR );
-    setMirrorType( 3, 2 );
+    createNewRole( 3, "CenterBack", Formation::SYNMETRY );
+    setSynmetryType( 3, 2 );
     createNewRole( 4, "SideBack", Formation::SIDE );
-    createNewRole( 5, "SideBack", Formation::MIRROR );
-    setMirrorType( 5, 4 );
+    createNewRole( 5, "SideBack", Formation::SYNMETRY );
+    setSynmetryType( 5, 4 );
     createNewRole( 6, "DefensiveHalf", Formation::CENTER );
     createNewRole( 7, "OffensiveHalf", Formation::SIDE );
-    createNewRole( 8, "OffensiveHalf", Formation::MIRROR );
-    setMirrorType( 8, 7 );
+    createNewRole( 8, "OffensiveHalf", Formation::SYNMETRY );
+    setSynmetryType( 8, 7 );
     createNewRole( 9, "SideForward", Formation::SIDE );
-    createNewRole( 10, "SideForward", Formation::MIRROR );
-    setMirrorType( 10, 9 );
+    createNewRole( 10, "SideForward", Formation::SYNMETRY );
+    setSynmetryType( 10, 9 );
     createNewRole( 11, "CenterForward", Formation::CENTER );
 
     Snapshot snap;
@@ -304,14 +304,14 @@ FormationRBF::createDefaultParam()
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "CenterBack", Formation::CENTER );
     createNewRole( 3, "SideBack", Formation::SIDE );
-    setMirrorType( 4, 3 );
+    setSynmetryType( 4, 3 );
     createNewRole( 5, "DefensiveHalf", Formation::SIDE );
-    setMirrorType( 6, 5 );
+    setSynmetryType( 6, 5 );
     createNewRole( 7, "SideHalf", Formation::SIDE );
-    setMirrorType( 8, 7 );
+    setSynmetryType( 8, 7 );
     createNewRole( 9, "OffensiveHalf", Formation::CENTER );
     createNewRole( 10, "Forward", Formation::SIDE );
-    setMirrorType( 11, 10 );
+    setSynmetryType( 11, 10 );
 #endif
 }
 
@@ -319,7 +319,7 @@ FormationRBF::createDefaultParam()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 FormationRBF::setRoleName( const int unum,
                            const std::string & name )
@@ -341,7 +341,7 @@ FormationRBF::setRoleName( const int unum,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 std::string
 FormationRBF::getRoleName( const int unum ) const
 {
@@ -361,7 +361,7 @@ FormationRBF::getRoleName( const int unum ) const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 FormationRBF::createNewRole( const int unum,
                              const std::string & role_name,
@@ -385,7 +385,7 @@ FormationRBF::createNewRole( const int unum,
     }
     else
     {
-        // mirror
+        // synmetry
     }
 
     // erase old parameter, if exist
@@ -405,7 +405,7 @@ FormationRBF::createNewRole( const int unum,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 Vector2D
 FormationRBF::getPosition( const int unum,
                            const Vector2D & ball_pos ) const
@@ -420,8 +420,8 @@ FormationRBF::getPosition( const int unum,
         return Vector2D::INVALIDATED;
     }
     Formation::SideType type = Formation::SIDE;
-    if ( M_mirror_number[unum - 1] > 0 )  type = Formation::MIRROR;
-    if ( M_mirror_number[unum - 1] == 0 ) type = Formation::CENTER;
+    if ( M_synmetry_number[unum - 1] > 0 )  type = Formation::SYNMETRY;
+    if ( M_synmetry_number[unum - 1] == 0 ) type = Formation::CENTER;
 
     return ptr->getPosition( ball_pos, type );
 }
@@ -429,7 +429,7 @@ FormationRBF::getPosition( const int unum,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 FormationRBF::getPositions( const Vector2D & focus_point,
                             std::vector< Vector2D > & positions ) const
@@ -445,7 +445,7 @@ FormationRBF::getPositions( const Vector2D & focus_point,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 FormationRBF::train( const std::list< Snapshot > & train_data )
 {
@@ -573,7 +573,7 @@ FormationRBF::train( const std::list< Snapshot > & train_data )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 boost::shared_ptr< FormationRBF::Param >
 FormationRBF::getParam( const int unum )
 {
@@ -604,7 +604,7 @@ FormationRBF::getParam( const int unum )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 boost::shared_ptr< const FormationRBF::Param >
 FormationRBF::param( const int unum ) const
 {
@@ -635,7 +635,7 @@ FormationRBF::param( const int unum ) const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 FormationRBF::read( std::istream & is )
 {
@@ -655,19 +655,19 @@ FormationRBF::read( std::istream & is )
     }
 
     //---------------------------------------------------
-    // check mirror number circuration reference
+    // check synmetry number circuration reference
     for ( int i = 0; i < 11; ++i )
     {
-        int refered_unum = M_mirror_number[i];
+        int refered_unum = M_synmetry_number[i];
         if ( refered_unum <= 0 ) continue;
-        if ( M_mirror_number[refered_unum - 1] > 0 )
+        if ( M_synmetry_number[refered_unum - 1] > 0 )
         {
             std::cerr << __FILE__ << ":" << __LINE__
                       << " *** ERROR *** failed to read formation."
-                      << " Bad mirroring. player "
+                      << " Bad synmetrying. player "
                       << i + 1
                       << " mirro = " << refered_unum
-                      << " is already mirroring player"
+                      << " is already synmetrying player"
                       << std::endl;
             return false;
         }
@@ -679,7 +679,7 @@ FormationRBF::read( std::istream & is )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 FormationRBF::readPlayers( std::istream & is )
 {
@@ -697,11 +697,11 @@ FormationRBF::readPlayers( std::istream & is )
 
         // check id
         int unum = 0;
-        int mirror = 0;
+        int synmetry = 0;
         int n_read = 0;
         if ( std::sscanf( line_buf.c_str(),
                           " player %d %d %n ",
-                          &unum, &mirror,
+                          &unum, &synmetry,
                           &n_read ) != 2
              || n_read == 0 )
         {
@@ -721,38 +721,38 @@ FormationRBF::readPlayers( std::istream & is )
                       << std::endl;
             return false;
         }
-        if ( mirror == unum )
+        if ( synmetry == unum )
         {
             std::cerr << __FILE__ << ":" << __LINE__
                       << " *** ERROR *** failed to read formation."
-                      << " Invalid mirror number. at "
+                      << " Invalid synmetry number. at "
                       << i
                       << " mirroing player itself. unum = " << unum
-                      << "  mirror = " << mirror
+                      << "  synmetry = " << synmetry
                       << std::endl;
             return false;
         }
-        if ( 11 < mirror )
+        if ( 11 < synmetry )
         {
             std::cerr << __FILE__ << ":" << __LINE__
                       << " *** ERROR *** failed to read formation."
-                      << " Invalid mirror number. at "
+                      << " Invalid synmetry number. at "
                       << i
-                      << "  Mirror number is out of range. unum = " << unum
-                      << "  mirror = " << mirror
+                      << "  Synmetry number is out of range. unum = " << unum
+                      << "  synmetry = " << synmetry
                       << std::endl;
             return false;
         }
 
-        M_mirror_number[i] = mirror;
+        M_synmetry_number[i] = synmetry;
 
-        // this player is mirror type
+        // this player is synmetry type
         /*
-        if ( mirror > 0 )
-        {
-            ++player_read;
-            continue;
-        }
+          if ( synmetry > 0 )
+          {
+          ++player_read;
+          continue;
+          }
         */
 
         // read parameters
@@ -801,13 +801,13 @@ FormationRBF::print( std::ostream & os ) const
     for ( int i = 0; i < 11; ++i )
     {
         const int unum = i + 1;
-        os << "player " << unum << " " << M_mirror_number[i] << " ";
+        os << "player " << unum << " " << M_synmetry_number[i] << " ";
         /*
-        if ( M_mirror_number[i] > 0 )
-        {
-            os << '\n';
-            continue;
-        }
+          if ( M_synmetry_number[i] > 0 )
+          {
+          os << '\n';
+          continue;
+          }
         */
         os << '\n';
 
