@@ -56,14 +56,14 @@ FormationStatic::createDefaultParam()
 {
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "CenterBack", Formation::SIDE );
-    setMirrorType( 3, 2 );
+    setSynmetryType( 3, 2 );
     createNewRole( 4, "SideBack", Formation::SIDE );
-    setMirrorType( 5, 4 );
+    setSynmetryType( 5, 4 );
     createNewRole( 6, "DefensiveHalf", Formation::CENTER );
     createNewRole( 7, "OffensiveHalf", Formation::SIDE );
-    setMirrorType( 8, 7 );
+    setSynmetryType( 8, 7 );
     createNewRole( 9, "SideForward", Formation::SIDE );
-    setMirrorType( 10, 9 );
+    setSynmetryType( 10, 9 );
     createNewRole( 11, "CenterForward", Formation::CENTER );
 
     Snapshot snapshot;
@@ -110,7 +110,7 @@ FormationStatic::createNewRole( const int unum,
     case Formation::SIDE:
         setSideType( unum );
         break;
-    case Formation::MIRROR:
+    case Formation::SYNMETRY:
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** Invalid side type "
                   << std::endl;
