@@ -157,6 +157,16 @@ public:
                       Vector2D * sol1,
                       Vector2D * sol2 ) const;
 
+    /*!
+      \brief calculate the intersection with another circle
+      \param circle considerd circle
+      \param sol1 pointer to the 1st solution variable
+      \param sol2 pointer to the 2nd solution variable
+      \return the number of solution
+     */
+    int intersection( const Circle2D & circle,
+                      Vector2D * sol1,
+                      Vector2D * sol2 ) const;
 
     // static utility
 
