@@ -15,7 +15,7 @@
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
 
- This library is distributed in the hope that it will be useful,
+ this library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
@@ -242,7 +242,7 @@ DelaunayTriangulation::createInitialTriangle()
     {
         if ( vit->pos().x < min_x ) min_x = vit->pos().x;
         else if ( max_x < vit->pos().x ) max_x = vit->pos().x;
-        if ( vit->pos().x < min_y ) min_y = vit->pos().y;
+        if ( vit->pos().y < min_y ) min_y = vit->pos().y;
         else if ( max_y < vit->pos().y ) max_y = vit->pos().y;
     }
 
