@@ -283,7 +283,7 @@ gzfilebuf::makeModeString( std::ios_base::openmode mode,
          || ( ! testi && ! testo ) )
     {
         // invalid mode
-        return std::string("");
+        return std::string( "" );
     }
 
     if ( testi )
@@ -314,7 +314,7 @@ gzfilebuf::makeModeString( std::ios_base::openmode mode,
     else
     {
         // invalid compression level
-        return std::string("");
+        return std::string( "" );
     }
 
     // set strategy
@@ -408,6 +408,9 @@ gzfilebuf::seekoff( std::streamoff off,
         if ( way & std::ios_base::beg )
         {
             ret = gzseek( M_impl->file_, off, SEEK_SET );
+            //std::cerr << "seekoff in beg off = " << off
+            //          << " ret = " << ret
+            //          << std::endl;
             M_remained_size = 0;
             this->setg( M_buf, M_buf, M_buf );
         }
@@ -415,6 +418,13 @@ gzfilebuf::seekoff( std::streamoff off,
         if ( way & std::ios_base::cur )
         {
             ret = gzseek( M_impl->file_, off, SEEK_CUR );
+            ret -= this->egptr() - this->gptr();
+            //off_type diff = this->egptr() - this->gptr();
+            //std::cerr << "seekoff in cur off = " << off
+            //          << " ret = " << ret
+            //          << " pointer_diff = "<< diff
+            //          << " total = " << ret - diff
+            //          << std::endl;
             if ( off != 0 )
             {
                 M_remained_size = 0;
@@ -468,6 +478,7 @@ gzfilebuf::seekpos( std::streampos pos,
     if ( ( M_impl->open_mode_ & std::ios_base::in )
          && ( mode & std::ios_base::in ) )
     {
+        //std::cerr << "seekpos in " << pos << std::endl;
         ret = gzseek( M_impl->file_, pos, SEEK_SET );
         // and reset buffer pointer to initial position
         M_remained_size = 0;
@@ -477,6 +488,7 @@ gzfilebuf::seekpos( std::streampos pos,
     if ( ( M_impl->open_mode_ & std::ios_base::out )
          && ( mode & std::ios_base::out ) )
     {
+        //std::cerr << "seekpos out " << pos << std::endl;
         std::streampos cur = gztell( M_impl->file_ );
         if ( pos <= cur )
         {
@@ -576,7 +588,8 @@ gzfilebuf::underflow()
     {
         return traits_type::eof();
     }
-
+    //std::cerr << "read_pos = " << gzseek( M_impl->file_, 0, SEEK_CUR )
+    //          << std::endl;
     int total_size = read_size + M_remained_size;
     this->setg( M_buf, M_buf, M_buf + total_size / sizeof( char_type ) );
     //std::cerr << "gzfilebuf::underflow. total_size=" << total_size
