@@ -252,14 +252,26 @@ ParamMap::add( ParamPtr param )
         return M_registrar;
     }
 
+    if ( param->longName().find( '=' ) != std::string::npos
+         || param->shortName().find( '=' ) != std::string::npos )
+    {
+        std::cerr << " ***ERROR*** "
+                  << " the option name [" << param->longName()
+                  << "] or [" << param->shortName()
+                  << "] contains an illegal character \'=\'."
+                  << std::endl;
+        return M_registrar;
+    }
+
     if ( M_long_name_map.find( param->longName() ) != M_long_name_map.end()
          || M_short_name_map.find( param->shortName() ) != M_short_name_map.end() )
     {
         std::cerr << " ***ERROR*** "
                   << " the option name [" << param->longName()
                   << "] or [" << param->shortName()
-                  << "] is already registered by other map."
+                  << "] has already been registered."
                   << std::endl;
+        return M_registrar;
     }
 
     M_parameters.push_back( param );
@@ -306,7 +318,7 @@ ParamMap::remove( const std::string & long_name )
 
 */
 ParamPtr
-ParamMap::getParamPtr( const std::string & long_name )
+ParamMap::findLongName( const std::string & long_name )
 {
     std::map< std::string, ParamPtr >::iterator it
         = M_long_name_map.find( long_name );
@@ -317,7 +329,7 @@ ParamMap::getParamPtr( const std::string & long_name )
     }
 
     // return NULL
-    return ParamPtr( static_cast< ParamEntry * >( 0 ) );
+    return ParamPtr( static_cast< ParamEntity * >( 0 ) );
 }
 
 /*-------------------------------------------------------------------*/
@@ -325,7 +337,7 @@ ParamMap::getParamPtr( const std::string & long_name )
 
 */
 ParamPtr
-ParamMap::getParamPtrShortName( const std::string & short_name )
+ParamMap::findShortName( const std::string & short_name )
 {
     std::map< std::string, ParamPtr >::iterator it
         = M_short_name_map.find( short_name );
@@ -336,7 +348,7 @@ ParamMap::getParamPtrShortName( const std::string & short_name )
     }
 
     // return NULL
-    return ParamPtr( static_cast< ParamEntry * >( 0 ) );
+    return ParamPtr( static_cast< ParamEntity * >( 0 ) );
 }
 
 /*-------------------------------------------------------------------*/
