@@ -37,13 +37,13 @@
 
 #include "player_agent.h"
 #include "world_model.h"
-#include "logger.h"
 #include "body_sensor.h"
 #include "player_command.h"
 #include "say_message_builder.h"
 
 #include <rcsc/math_util.h>
 #include <rcsc/common/server_param.h>
+#include <rcsc/common/logger.h>
 
 namespace rcsc {
 
@@ -74,7 +74,6 @@ ActionEffector::ActionEffector( const PlayerAgent & agent )
     , M_tackle_power( 0.0 )
     , M_tackle_dir( 0.0 )
     , M_turn_neck_moment( 0.0 )
-    , M_protocol_version( 9.0 )
     , M_say_message( "" )
     , M_pointto_pos( 0.0, 0.0 )
 {
@@ -484,6 +483,7 @@ ActionEffector::setKick( const double & power,
                       command_power);
         command_power = ServerParam::i().maxPower();
     }
+
     if ( command_power < 0.0 )
     {
         std::cerr << M_agent.world().teamName() << ' '
@@ -849,7 +849,7 @@ ActionEffector::setMove( const double & x,
                           command_x, command_y );
             command_x = - ServerParam::i().pitchHalfLength() + 1.0;
         }
-        if ( command_x > - ServerParam::i().ourPenaltyAreaLine() - 1.0 )
+        if ( command_x > - ServerParam::i().ourPenaltyAreaLineX() - 1.0 )
         {
             std::cerr << M_agent.world().teamName() << ' '
                       << M_agent.world().self().unum() << ": "
@@ -859,7 +859,7 @@ ActionEffector::setMove( const double & x,
             dlog.addText( Logger::ACTION,
                           "Effector::setMove. (%.1f, %.1f). must move to a place in penalty area(2)",
                           command_x, command_y );
-            command_x = - ServerParam::i().ourPenaltyAreaLine() - 1.0;
+            command_x = - ServerParam::i().ourPenaltyAreaLineX() - 1.0;
         }
         if ( command_y > ServerParam::i().penaltyAreaHalfWidth() - 1.0 )
         {
@@ -939,39 +939,60 @@ ActionEffector::setCatch()
 
 */
 void
-ActionEffector::setTackle( const double & power )
+ActionEffector::setTackle( const double & power_or_dir )
 {
     dlog.addText( Logger::ACTION,
-                  "register tackle. power= %.1f",
-                  power );
+                  "register tackle. power_or_dir=%.1f",
+                  power_or_dir );
 
-    double command_power = power;
+    double command_arg = power_or_dir;
 
-    if ( command_power > ServerParam::i().maxPower() + 0.01 )
+    if ( M_agent.config().version() >= 12.0 )
     {
-        std::cerr << M_agent.world().teamName() << ' '
-                  << M_agent.world().self().unum() << ": "
-                  << M_agent.world().time()
-                  << "tackle power overflow. com=" << command_power
-                  << " > sparam=" << ServerParam::i().maxPower() << std::endl;
-        dlog.addText( Logger::ACTION,
-                      "Effector::setTackle. power over. %.10f",
-                      command_power);
-        command_power = ServerParam::i().maxPower();
+        if ( command_arg < -180.0 || 180.0 < command_arg )
+        {
+            std::cerr << M_agent.world().teamName() << ' '
+                      << M_agent.world().self().unum() << ": "
+                      << M_agent.world().time()
+                      << "tackle dir over the range. dir=" << command_arg
+                      << std::endl;
+            dlog.addText( Logger::ACTION,
+                          "Effector::setTackle. dir over. %f",
+                          command_arg );
+        }
+        command_arg = AngleDeg::normalize_angle( power_or_dir );
     }
-    if ( command_power < ServerParam::i().minPower() - 0.01 )
+    else
     {
-        std::cerr << M_agent.world().teamName() << ' '
-                  << M_agent.world().self().unum() << ": "
-                  << M_agent.world().time()
-                  << "tackle power underflow " << command_power << std::endl;
-        dlog.addText( Logger::ACTION,
-                      "Effector::setKick. power underflow. %.1f",
-                      command_power );
-        command_power = ServerParam::i().minPower();
+        if ( command_arg > ServerParam::i().maxTacklePower() + 0.01 )
+        {
+            std::cerr << M_agent.world().teamName() << ' '
+                      << M_agent.world().self().unum() << ": "
+                      << M_agent.world().time()
+                      << "tackle power overflow. com=" << command_arg
+                      << " > sparam=" << ServerParam::i().maxTacklePower()
+                      << std::endl;
+            dlog.addText( Logger::ACTION,
+                          "Effector::setTackle. power over. %f",
+                          command_arg );
+            command_arg = ServerParam::i().maxTacklePower();
+        }
+
+        if ( command_arg < - ServerParam::i().maxBackTacklePower() - 0.01 )
+        {
+            std::cerr << M_agent.world().teamName() << ' '
+                      << M_agent.world().self().unum() << ": "
+                      << M_agent.world().time()
+                      << "tackle power underflow " << command_arg
+                      << std::endl;
+            dlog.addText( Logger::ACTION,
+                          "Effector::setKick. power underflow. %f",
+                          command_arg );
+            command_arg = ServerParam::i().minPower();
+        }
     }
 
-    command_power = rint( command_power );
+    command_arg = rint( command_arg );
 
     //////////////////////////////////////////////////
     // create command object
@@ -980,13 +1001,21 @@ ActionEffector::setTackle( const double & power )
         delete M_command_body;
         M_command_body = static_cast< PlayerBodyCommand * >( 0 );
     }
-    M_command_body = new PlayerTackleCommand( power );
+    M_command_body = new PlayerTackleCommand( command_arg );
 
     // set estimated command effect
-    M_tackle_power = command_power;
-    M_tackle_dir = ( command_power > 0.0
-                     ? M_agent.world().self().body().degree()
-                     : ( M_agent.world().self().body() + 180.0 ).degree() );
+    M_tackle_power = command_arg;
+    if ( M_agent.config().version() >= 12.0 )
+    {
+        M_tackle_power = ServerParam::i().maxTacklePower();
+        M_tackle_dir = command_arg;
+    }
+    else
+    {
+        M_tackle_dir = ( command_arg > 0.0
+                         ? M_agent.world().self().body().degree()
+                         : ( M_agent.world().self().body() + 180.0 ).degree() );
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -1083,8 +1112,10 @@ ActionEffector::setChangeView( const ViewWidth & width )
         delete M_command_change_view;
         M_command_change_view = static_cast< PlayerChangeViewCommand * >( 0 );
     }
+
     M_command_change_view = new PlayerChangeViewCommand( width,
-                                                         ViewQuality::HIGH );
+                                                         ViewQuality::HIGH,
+                                                         M_agent.config().version() );
 }
 
 /*-------------------------------------------------------------------*/
@@ -1176,6 +1207,7 @@ ActionEffector::setPointto( const double & x,
 
     Vector2D target_pos( x, y );
     Vector2D target_rel = target_pos - M_agent.world().self().pos();
+    target_rel.rotate( - M_agent.world().self().face() );
 
     //////////////////////////////////////////////////
     // create command object
