@@ -114,7 +114,6 @@ private:
     double M_turn_neck_moment; //!< last turn_neck moment
 
     // say effect
-    double M_protocol_version; //!< client protocl version
     std::string M_say_message; //!< last said message string
     std::vector< const SayMessage * > M_say_messages;
 
@@ -280,11 +279,11 @@ public:
 
     /*!
       \brief create tackle command and its effect with tackle parameter
-      \param power tackle power for command argument
+      \param power_or_dir tackle power or directin for command argument
 
       power is normalized by server parameter
     */
-    void setTackle( const double & power );
+    void setTackle( const double & power_or_dir );
 
     ///////////////////////////////////////////////////////////////
     // register support command
