@@ -71,27 +71,43 @@ private:
     Vector2D M_pos;
     //! estimated global coordinate error
     Vector2D M_pos_error;
-    //! global position accuracy. count from last recognition
+    //! cycle count since last observation
     int M_pos_count;
 
     //! estimated position relative to self
     Vector2D M_rpos;
     //! estimated error of relative coordinate
     Vector2D M_rpos_error;
-    //! relative position accuracy. count from last relative pos observation
+    //! cycle count since last relative pos observation
     int M_rpos_count;
     //! relative position at previous cycle. updated only in pure internal update
     Vector2D M_rpos_prev;
 
+    //! last seen global position
+    Vector2D M_seen_pos;
+    //! cycle count since last observation
+    int M_seen_pos_count;
+    //! previous seen relative position
+    Vector2D M_seen_rpos;
+
+    //! last heard global position
+    Vector2D M_heard_pos;
+    //! cycle count since last observation
+    int M_heard_pos_count;
+
     //! estimated global velocity
     Vector2D M_vel;
     //! estimated error of global velocity
     Vector2D M_vel_error;
-    //! velocity accuracy. count from last observation
+    //! velocity accuracy. count since last observation
     int M_vel_count;
 
-    Vector2D M_last_seen_pos; //!< last seen global coordinate
-    //! count from last see observation
+    //! last heard velocity
+    Vector2D M_heard_vel;
+    //! cycle count since last observation
+    int M_heard_vel_count;
+
+    //! count since ball lost
     int M_lost_count;
 
     //! last ghost check time
@@ -207,6 +223,54 @@ public:
           return M_rpos_prev;
       }
 
+    /*!
+      \brief get the last seen position
+      \return const reference to the point object
+     */
+    const
+    Vector2D & seenPos() const
+      {
+          return M_seen_pos;
+      }
+
+    /*!
+      \brief get the number of cycles since last seen
+      \return count since last seen
+    */
+    int seenPosCount() const
+      {
+          return M_seen_pos_count;
+      }
+
+    /*!
+      \brief get the last seen relative position
+      \return const reference to the variable
+     */
+    const
+    Vector2D & seenRPos() const
+      {
+          return M_seen_rpos;
+      }
+
+    /*!
+      \brief get the last heard position
+      \return const reference to the point object
+     */
+    const
+    Vector2D & heardPos() const
+      {
+          return M_heard_pos;
+      }
+
+    /*!
+      \brief get the number of cycles since last observation
+      \return count since last observation
+    */
+    int heardPosCount() const
+      {
+          return M_heard_pos_count;
+      }
+
     /*!
       \brief get estimated velocity
       \return const referenct to vector object
@@ -237,18 +301,8 @@ public:
       }
 
     /*!
-      \brief get last observed position
-      \return const referenct to vector object
-    */
-    const
-    Vector2D & lastSeenPos() const
-      {
-          return M_last_seen_pos;
-      }
-
-    /*!
-      \brief get lost count
-      \return count from last observation
+      \brief get count since ball lost
+      \return cycle count since last observation
     */
     int lostCount() const
       {
@@ -356,9 +410,11 @@ public:
       \brief update velocity using see info
       \param vel estimated velocity
       \param vel_err estimated error of velocity
+      \param vel_count new accuracy value of the velocity
     */
     void updateOnlyVel( const Vector2D & vel,
-                        const Vector2D & vel_err );
+                        const Vector2D & vel_err,
+                        const int vel_count );
 
     /*!
       \brief update position by see info (not include velocity)
@@ -385,6 +441,7 @@ public:
       \param rpos_err estimated error of relative position
       \param vel estimated velocity
       \param vel_err estimated error of velocity
+      \param vel_count new accuracy value of the velocity
     */
     void updateAll( const Vector2D & pos,
                     const Vector2D & pos_err,
@@ -392,7 +449,8 @@ public:
                     const Vector2D & rpos,
                     const Vector2D & rpos_err,
                     const Vector2D & vel,
-                    const Vector2D & vel_err );
+                    const Vector2D & vel_err,
+                    const int vel_count );
 
     /*!
       \brief update ball status using heared info
