@@ -38,8 +38,8 @@
 #include "player_intercept.h"
 #include "world_model.h"
 #include "player_object.h"
-#include "logger.h"
 
+#include <rcsc/common/logger.h>
 #include <rcsc/common/server_param.h>
 #include <rcsc/game_time.h>
 
@@ -207,7 +207,20 @@ InterceptTable::hearTeammate( const int unum,
         return;
     }
 
-    const PlayerObject * p = M_world.getTeammate( unum );
+    const PlayerObject * p = static_cast< PlayerObject * >( 0 );
+
+    const PlayerPtrCont::const_iterator end = M_world.teammatesFromSelf().end();
+    for ( PlayerPtrCont::const_iterator it = M_world.teammatesFromSelf().begin();
+          it != end;
+          ++it )
+    {
+        if ( (*it)->unum() == unum )
+        {
+            p = *it;
+            break;
+        }
+    }
+
     if ( p )
     {
         M_fastest_teammate = p;
@@ -255,7 +268,20 @@ InterceptTable::hearOpponent( const int unum,
         }
     }
 
-    const PlayerObject * p = M_world.getOpponent( unum );
+    const PlayerObject * p = static_cast< PlayerObject * >( 0 );
+
+    const PlayerPtrCont::const_iterator end = M_world.opponentsFromSelf().end();
+    for ( PlayerPtrCont::const_iterator it = M_world.opponentsFromSelf().begin();
+          it != end;
+          ++it )
+    {
+        if ( (*it)->unum() == unum )
+        {
+            p = *it;
+            break;
+        }
+    }
+
     if ( p )
     {
         M_fastest_opponent = p;
