@@ -32,7 +32,7 @@
 #ifndef RCSC_PLAYER_INTERCEPTION_H
 #define RCSC_PLAYER_INTERCEPTION_H
 
-#include <rcsc/param/player_type.h>
+#include <rcsc/common/player_type.h>
 #include <rcsc/geom/vector_2d.h>
 
 namespace rcsc {
