@@ -40,17 +40,16 @@
 #include "audio_sensor.h"
 #include "fullstate_sensor.h"
 
-#include "audio_memory.h"
 #include "debug_client.h"
 #include "freeform_parser.h"
-#include "logger.h"
 #include "player_command.h"
 #include "say_message_builder.h"
-#include "say_message_parser.h"
 #include "soccer_action.h"
 #include "soccer_intention.h"
 
+#include <rcsc/common/audio_memory.h>
 #include <rcsc/common/basic_client.h>
+#include <rcsc/common/logger.h>
 #include <rcsc/common/player_param.h>
 #include <rcsc/common/player_type.h>
 #include <rcsc/common/server_param.h>
@@ -183,42 +182,47 @@ PlayerAgentImpl::updateCurrentTime( const long & new_time,
 {
     // called after sense (see, hear, sense_body, fullstate...)
 
+    GameTime old_time = current_time_;
+
     // server cycle stopped (BeforeKickOff, Offside, BackPass, FreeKickFault)
     if ( server_cycle_stopped_ )
     {
         if ( new_time != current_time_.cycle() )
         {
+            current_time_.assign( new_time, 0 ) ;
+
             dlog.addText( Logger::LEVEL_ANY,
                           "CYCLE %ld-0 --------------------"
                           " return from cycle stop",
                           new_time );
-            if ( new_time - 1 != current_time_.cycle() )
+            if ( new_time - 1 != old_time.cycle() )
             {
                 std::cout << agent_.config().teamName() << ' '
                           << agent_.world().self().unum() << ": "
                           << current_time_
                           << " Stop Mode: previous server time is incorrect?? "
-                          << current_time_ << " -> " << new_time << std::endl;
+                          << old_time << " -> " << new_time << std::endl;
                 dlog.addText( Logger::SYSTEM,
                               "System: stop mode: previous server time is incorrect??  (%ld, %ld) -> %ld",
-                              current_time_.cycle(), current_time_.stopped(),
+                              old_time.cycle(), old_time.stopped(),
                               new_time );
             }
-
-            current_time_.assign( new_time, 0 ) ;
         }
         else
         {
             // if sense type is sense_body, it can be updated very safety.
             if ( by_sense_body )
             {
+                current_time_.assign( current_time_.cycle(),
+                                      current_time_.stopped() + 1 );
+
                 dlog.addText( Logger::LEVEL_ANY,
                               "CYCLE %ld-%ld --------------------"
                               " stopped time was updated by sense_body",
                               current_time_.cycle(), current_time_.stopped() + 1 );
-                if ( last_decision_time_ != current_time_ )
+                if ( last_decision_time_ != old_time )
                 {
-                    if ( current_time_.stopped() == 0 )
+                    if ( old_time.stopped() == 0 )
                     {
                         // just after changed to stop mode.
                         //   e.g. after goal playmode
@@ -236,29 +240,29 @@ PlayerAgentImpl::updateCurrentTime( const long & new_time,
                                   << std::endl;
                     }
                 }
-                current_time_.assign( current_time_.cycle(),
-                                      current_time_.stopped() + 1 );
             }
         }
     }
     // normal case
     else
     {
-        if ( current_time_.cycle() != new_time )
+        current_time_.assign( new_time, 0 );
+
+        if ( old_time.cycle() != new_time )
         {
             dlog.addText( Logger::LEVEL_ANY,
                           "CYCLE %ld-0  -------------------------------------------------",
                           new_time );
-            if ( new_time - 1 != current_time_.cycle() )
+            if ( new_time - 1 != old_time.cycle() )
             {
                 std::cout << agent_.config().teamName() << ' '
                           << agent_.world().self().unum() << ": "
                           << current_time_
                           << " skipped server time?? "
-                          << current_time_ << " -> " << new_time << std::endl;
+                          << old_time << " -> " << new_time << std::endl;
                 dlog.addText( Logger::SYSTEM,
                               "skipped server time?? (%ld, %ld) -> %ld",
-                              current_time_.cycle(), current_time_.stopped(),
+                              old_time.cycle(), old_time.stopped(),
                               new_time );
             }
 
@@ -275,7 +279,6 @@ PlayerAgentImpl::updateCurrentTime( const long & new_time,
                           << std::endl;
             }
         }
-        current_time_.assign( new_time, 0 );
     }
 }
 
@@ -322,6 +325,12 @@ PlayerAgentImpl::isDecisionTiming( const long & msec_from_sense ) const
         return false;
     }
 
+    // not initialized
+    if ( agent_.world().self().unum() == Unum_Unknown )
+    {
+        return false;
+    }
+
     // got current see info
     if ( agent_.world().seeTime() == current_time_ )
     {
@@ -381,27 +390,8 @@ PlayerAgent::PlayerAgent()
 {
     // std::cerr << "construct player" << std::endl;
 
-    typedef boost::shared_ptr< SayMessageParser > SMP;
-    typedef boost::shared_ptr< FreeformParser > FP;
-
     boost::shared_ptr< AudioMemory > audio_memory( new AudioMemory );
-
     M_worldmodel.setAudioMemory( audio_memory );
-
-    addSayMessageParser( SMP( new BallMessageParser( audio_memory ) ) );
-    addSayMessageParser( SMP( new PassMessageParser( audio_memory ) ) );
-    addSayMessageParser( SMP( new InterceptMessageParser( audio_memory ) ) );
-    addSayMessageParser( SMP( new GoalieMessageParser( audio_memory ) ) );
-    addSayMessageParser( SMP( new OffsideLineMessageParser( audio_memory ) ) );
-    addSayMessageParser( SMP( new DefenseLineMessageParser( audio_memory ) ) );
-    addSayMessageParser( SMP( new WaitRequestMessageParser( audio_memory ) ) );
-    addSayMessageParser( SMP( new PassRequestMessageParser( audio_memory ) ) );
-    addSayMessageParser( SMP( new DribbleMessageParser( audio_memory ) ) );
-    addSayMessageParser( SMP( new BallGoalieMessageParser( audio_memory ) ) );
-    addSayMessageParser( SMP( new OnePlayerMessageParser( audio_memory ) ) );
-    addSayMessageParser( SMP( new BallPlayerMessageParser( audio_memory ) ) );
-
-    M_impl->audio_.addParser( FP( new FreeformParser( M_worldmodel ) ) );
 }
 
 /*-------------------------------------------------------------------*/
@@ -530,7 +520,7 @@ PlayerAgent::initImpl( CmdLineParser & cmd_parser )
     cmd_parser.parse( player_param_map );
 
     if ( config().version() < 8.0
-         || 12.0 <= config().version() )
+         || 13.0 <= config().version() )
     {
         std::cerr << "Unsupported client version: " << config().version()
                   << std::endl;
@@ -741,7 +731,6 @@ PlayerAgent::addSayMessageParser( boost::shared_ptr< SayMessageParser > parser )
     M_impl->audio_.addParser( parser );
 }
 
-
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -749,7 +738,17 @@ PlayerAgent::addSayMessageParser( boost::shared_ptr< SayMessageParser > parser )
 void
 PlayerAgent::removeSayMessageParser( const char header )
 {
-    M_impl->audio_.removeSayMessageParser( header );
+    M_impl->audio_.removeParser( header );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+PlayerAgent::setFreeformParser( boost::shared_ptr< FreeformParser > parser )
+{
+    M_impl->audio_.setFreeformParser( parser );
 }
 
 /*-------------------------------------------------------------------*/
@@ -849,19 +848,19 @@ PlayerAgent::setDebugFlags()
         return;
     }
 
-    dlog.setLogFlag( &world().time(), Logger::SYSTEM, config().debugSystem() );
-    dlog.setLogFlag( &world().time(), Logger::SENSOR, config().debugSensor() );
-    dlog.setLogFlag( &world().time(), Logger::WORLD, config().debugWorld() );
-    dlog.setLogFlag( &world().time(), Logger::ACTION, config().debugAction() );
-    dlog.setLogFlag( &world().time(), Logger::INTERCEPT, config().debugIntercept() );
-    dlog.setLogFlag( &world().time(), Logger::KICK, config().debugKick() );
-    dlog.setLogFlag( &world().time(), Logger::DRIBBLE, config().debugDribble() );
-    dlog.setLogFlag( &world().time(), Logger::PASS, config().debugPass() );
-    dlog.setLogFlag( &world().time(), Logger::CROSS, config().debugCross() );
-    dlog.setLogFlag( &world().time(), Logger::SHOOT, config().debugShoot() );
-    dlog.setLogFlag( &world().time(), Logger::CLEAR, config().debugClear() );
-    dlog.setLogFlag( &world().time(), Logger::TEAM, config().debugTeam() );
-    dlog.setLogFlag( &world().time(), Logger::ROLE, config().debugRole() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::SYSTEM, config().debugSystem() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::SENSOR, config().debugSensor() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::WORLD, config().debugWorld() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::ACTION, config().debugAction() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::INTERCEPT, config().debugIntercept() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::KICK, config().debugKick() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::DRIBBLE, config().debugDribble() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::PASS, config().debugPass() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::CROSS, config().debugCross() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::SHOOT, config().debugShoot() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::CLEAR, config().debugClear() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::TEAM, config().debugTeam() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::ROLE, config().debugRole() );
 }
 
 /*-------------------------------------------------------------------*/
@@ -949,7 +948,7 @@ PlayerAgent::analyzeCycle( const char * msg,
 {
     char id[16];
     long cycle = 0;
-    if ( std::sscanf( msg, "(%s %ld ",
+    if ( std::sscanf( msg, "(%15s %ld ",
                       id, &cycle ) != 2 )
     {
         std::cerr << world().teamName() << ' '
@@ -992,16 +991,16 @@ PlayerAgent::analyzeSee( const char * msg )
 #endif
     }
 
-    dlog.addText( Logger::SENSOR,
-                  "===receive see --- [%ld]ms from sense_body",
-                  msec_from_sense );
-
     // parse cycle info
     if ( ! analyzeCycle( msg, false ) )
     {
         return;
     }
 
+    dlog.addText( Logger::SENSOR,
+                  "===receive see --- [%ld]ms from sense_body",
+                  msec_from_sense );
+
     // parse see info
     M_impl->visual_.parse( msg,
                            config().teamName().c_str(),
@@ -1052,23 +1051,29 @@ PlayerAgent::analyzeSee( const char * msg )
   (sense_body <Time> <BodyInfo> <BodyInfo> ...
 */
 void
-PlayerAgent::analyzeSenseBody( const char* msg )
+PlayerAgent::analyzeSenseBody( const char * msg )
 {
     M_impl->body_time_stamp_.setCurrent();
-    // analyze process
-    dlog.addText( Logger::SENSOR,
-                  "===receive sense_body" );
+
     // parse cycle info
     if ( ! analyzeCycle( msg, true ) )
     {
         return;
     }
+
+    // analyze process
+    dlog.addText( Logger::SENSOR,
+                  "===receive sense_body" );
+
     // parse body info
     M_impl->body_.parse( msg,
                          config().version(),
                          M_impl->current_time_ );
     // update see sync information
-    M_impl->see_state_.setNewCycle( M_impl->current_time_ );
+    //    M_impl->see_state_.setNewCycle( M_impl->current_time_ );
+    M_impl->see_state_.updateBySenseBody( M_impl->current_time_,
+                                          M_impl->body_.viewWidth(),
+                                          M_impl->body_.viewQuality() );
 
     //----------------------------------------------
     // update process
@@ -1258,7 +1263,7 @@ PlayerAgent::analyzeHearPlayer(  const char * msg )
     //----------------------------------------------
     // analyze process
     dlog.addText( Logger::SENSOR,
-                  "receive hear [%s]",
+                  "===receive hear [%s]",
                   msg );
 
     if ( ! config().useCommunication() )
@@ -1283,7 +1288,6 @@ PlayerAgent::analyzeHearOurCoach( const char * msg )
     M_impl->audio_.parseCoachMessage( msg, M_impl->current_time_ );
 }
 
-
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -1295,7 +1299,6 @@ PlayerAgent::analyzeHearOpponentCoach( const char * msg )
                   "===receive say message from opponent coach" );
 }
 
-
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -1316,14 +1319,14 @@ PlayerAgent::analyzeHearTrainer( const char * msg )
 void
 PlayerAgent::analyzeFullstate( const char * msg )
 {
-    dlog.addText( Logger::SENSOR,
-                  "===receive fullstate" );
-
     if ( ! analyzeCycle( msg, false ) )
     {
         return;
     }
 
+    dlog.addText( Logger::SENSOR,
+                  "===receive fullstate" );
+
     M_impl->fullstate_.parse( msg,
                               config().version(),
                               M_impl->current_time_ );
@@ -1385,7 +1388,7 @@ PlayerAgent::analyzeServerParam( const char * msg )
 
     // update alarm interval
     if ( ! ServerParam::i().synchMode()
-         && ServerParam::i().slowDownFactor() != 1 )
+         && ServerParam::i().slowDownFactor() > 1 )
     {
         long interval = ( config().intervalMSec()
                           * ServerParam::i().slowDownFactor() );
@@ -1518,9 +1521,7 @@ PlayerAgent::analyzeInit( const char * msg )
         }
         ostrm << config().teamName() << "-" << myunum
               << config().logExt();
-        std::string filepath = ostrm.str();
-        //std::cerr << unum() << " Debug File is [" << filepath << "]" << std::endl;
-        dlog.open( filepath.c_str() );
+        dlog.open( ostrm.str().c_str() );
         if ( ! dlog.isOpen() )
         {
             std::cerr << config().teamName() << ' '
@@ -1538,6 +1539,12 @@ PlayerAgent::analyzeInit( const char * msg )
         //////////////////////////
         // these must be moved to before kick off action ??
 
+        // set synch see mode
+        if ( config().synchSee() )
+        {
+            ostrm << "(synch_see)";
+        }
+
         // turn off opponent all audio message
 
         if ( ! config().hearOpponentAudio() )
@@ -1600,12 +1607,13 @@ PlayerAgent::analyzeChangePlayerType( const char * msg )
                   "parse change_player_type [%s]",
                   msg );
 
-    if ( std::sscanf( msg, "(change_player_type %d %d)", &unum, &type ) == 2 )
+    if ( std::sscanf( msg, " ( change_player_type %d %d ) ",
+                      &unum, &type ) == 2 )
     {
-        HeteroID id = static_cast< HeteroID >( type );
-        M_worldmodel.setTeammatePlayerType( unum, id );
+        M_worldmodel.setTeammatePlayerType( unum, type );
     }
-    else if ( std::sscanf( msg, "(change_player_type %d)", &unum ) == 1 )
+    else if ( std::sscanf( msg, " ( change_player_type %d ) ",
+                           &unum ) == 1 )
     {
         M_worldmodel.setOpponentPlayerType( unum, Hetero_Unknown );
     }
@@ -1631,6 +1639,20 @@ PlayerAgent::analyzeOK( const char * msg )
                   "===receive ok [%s]",
                   msg );
 
+    if ( ! std::strncmp( msg,
+                         "(ok synch_see)",
+                         std::strlen( "(ok synch_see)" ) ) )
+    {
+        std::cerr << world().teamName() << ' '
+                  << world().self().unum() << ": "
+                  << world().time()
+                  << " set synch see mode."
+                  << std::endl;
+        M_impl->see_state_.setSynchSeeMode();
+        M_impl->see_state_.setViewMode( world().self().viewWidth(),
+                                        world().self().viewQuality() );
+        return;
+    }
     if ( ! std::strncmp( msg,
                          "(ok compression ",
                          std::strlen( "(ok compression " ) ) )
@@ -2066,7 +2088,7 @@ PlayerAgent::outputDebug()
                                           world().self().pos().y + 15.0 ) );
         // compose worldmodel & some debug message
         // send to debug server or write to disc
-        M_debug_client.output( world() );
+        M_debug_client.writeAll( world() );
     }
     else
     {
@@ -2262,7 +2284,7 @@ PlayerAgent::doCatch()
 
 */
 bool
-PlayerAgent::doTackle( const double & power )
+PlayerAgent::doTackle( const double & power_or_dir )
 {
     if ( world().self().isFreezed() )
     {
@@ -2276,7 +2298,7 @@ PlayerAgent::doTackle( const double & power )
         return false;
     }
 
-    M_effector.setTackle( power );
+    M_effector.setTackle( power_or_dir );
     return true;
 }
 
