@@ -34,8 +34,8 @@
 #endif
 
 #include "player_config.h"
-#include "logger.h"
 
+#include <rcsc/common/logger.h>
 #include <rcsc/param/param_map.h>
 
 #include <cassert>
@@ -69,7 +69,7 @@ void
 PlayerConfig::setDefaultParam()
 {
     // basic setting
-    M_team_name = "HELIOS2007";
+    M_team_name = "HELIOS_base";
     M_version = 9.4;
     M_reconnect_number = 0;
     M_goalie = false;
@@ -95,6 +95,8 @@ PlayerConfig::setDefaultParam()
 
     M_use_fullstate = true;
 
+    M_synch_see = false;
+
     // accuracy threshold
     M_self_pos_count_thr = 20;
     M_self_vel_count_thr = 10;
@@ -113,8 +115,9 @@ PlayerConfig::setDefaultParam()
     M_player_number = 0;
 
 
-    //////////////////////////////////////////////////////
+    //
     // debug
+    //
     M_debug_connect = false;
     M_debug_server_host = "localhost";
     M_debug_server_port = 6000 + 32;
@@ -175,6 +178,7 @@ PlayerConfig::createParamMap( ParamMap & param_map )
         ( "hear_opponent_audio", "", &M_hear_opponent_audio )
 
         ( "use_fullstate", "", &M_use_fullstate )
+        ( "synch_see", "", &M_synch_see )
 
         ( "self_pos_count_thr", "", &M_self_pos_count_thr )
         ( "self_vel_count_thr", "", &M_self_vel_count_thr )
@@ -211,7 +215,8 @@ PlayerConfig::createParamMap( ParamMap & param_map )
         ( "debug_shoot", "", BoolSwitch( &M_debug_shoot ) )
         ( "debug_clear", "", BoolSwitch( &M_debug_clear ) )
         ( "debug_team", "", BoolSwitch( &M_debug_team ) )
-        ( "debug_role", "", BoolSwitch( &M_debug_role ) );
+        ( "debug_role", "", BoolSwitch( &M_debug_role ) )
+        ;
 }
 
 }
