@@ -37,10 +37,10 @@
 #include "world_model.h"
 #include "ball_object.h"
 #include "player_object.h"
-#include "logger.h"
 
-#include <rcsc/param/server_param.h>
-#include <rcsc/param/player_type.h>
+#include <rcsc/common/logger.h>
+#include <rcsc/common/server_param.h>
+#include <rcsc/common/player_type.h>
 #include <rcsc/soccer_math.h>
 
 namespace rcsc {
@@ -170,7 +170,7 @@ PlayerIntercept::predictTurnCycle( const int cycle,
                                    const double & control_area,
                                    const Vector2D & ball_pos ) const
 {
-    if ( player.bodyCount() > 0 )
+    if ( player.bodyCount() >= 4 )
     {
         return 0;
     }
@@ -219,7 +219,7 @@ PlayerIntercept::canReachAfterDash( const int n_turn,
         = player_type.inertiaPoint( player.pos(), player.vel(), n_turn );
     Vector2D tmp_pos = first_pos;
     Vector2D tmp_vel( 0.0, 0.0 ); // if n_turn > 0 , assume that vel == 0
-    if ( player.velCount() == 0 )
+    if ( player.velCount() < 4 )
     {
         tmp_vel = player.vel();
     }
@@ -237,7 +237,9 @@ PlayerIntercept::canReachAfterDash( const int n_turn,
     const Vector2D dash_accel
         = Vector2D::polar2vector( dash_accel_mag, dash_angle );
 
-    for ( int i = 0; i < n_dash; ++i )
+    const int total_dash = n_dash + std::min( 3, player.posCount() - n_turn );
+
+    for ( int i = 0; i < total_dash; ++i )
     {
         tmp_vel += dash_accel;
         // reach max speed
@@ -246,12 +248,12 @@ PlayerIntercept::canReachAfterDash( const int n_turn,
             double required_dist
                 = ( ball_pos - tmp_pos ).r() - control_area;
             if ( required_dist
-                 < player_type.realSpeedMax() * ( n_dash - i ) )
+                 < player_type.realSpeedMax() * ( total_dash - i ) )
             {
 #ifdef DEBUG
                 dlog.addText( Logger::INTERCEPT,
                               "________run over by %d dash... p(%f %f) dash_dist=%f",
-                              n_dash - i, tmp_pos.x, tmp_pos.y, required_dist );
+                              total_dash - i, tmp_pos.x, tmp_pos.y, required_dist );
 #endif
                 return true;
             }
@@ -268,7 +270,7 @@ PlayerIntercept::canReachAfterDash( const int n_turn,
 #ifdef DEBUG
         dlog.addText( Logger::INTERCEPT,
                       "______dash finised %d. run over... b(%f %f) p(%f %f)",
-                      n_dash,
+                      total_dash,
                       ball_pos.x, ball_pos.y,
                       tmp_pos.x, tmp_pos.y );
 #endif
