@@ -32,6 +32,8 @@
 #ifndef RCSC_PLAYER_PLAYER_OBJECT_H
 #define RCSC_PLAYER_PLAYER_OBJECT_H
 
+#include <rcsc/player/abstract_player_object.h>
+
 #include <rcsc/player/localization.h>
 #include <rcsc/player/fullstate_sensor.h>
 
@@ -43,13 +45,12 @@
 
 namespace rcsc {
 
-class PlayerType;
-
 /*!
   \class PlayerObject
   \brief observed player object class
 */
-class PlayerObject {
+class PlayerObject
+    : public AbstractPlayerObject {
 private:
     //! validation count threshold value for M_pos and M_rpos
     static int S_pos_count_thr;
@@ -58,35 +59,14 @@ private:
     //! validation count threshold value for M_body and M_face
     static int S_face_count_thr;
 
-    int  M_unum; //!< uniform number
-    bool M_goalie; //!< goalie flag
-
-    HeteroID M_type; //!< player type id
-
-    const PlayerType * M_player_type_ptr; //!< player type reference
-
     double M_dist_from_self; //!< distance from self
     AngleDeg M_angle_from_self; //!< angle from self global
-    double M_dist_from_ball; //!< distance from ball
 
     int M_ghost_count; //!< ghost flag
 
-    Vector2D M_pos; //!< global coordinate
     Vector2D M_rpos; //!< relative coordinate
-    int M_pos_count; //!< main accuracy counter
     int M_rpos_count; //!< relative coordinate accuracy counter
 
-    Vector2D M_prev_pos; //! previous seen global coordinate
-    int M_prev_pos_count; //! count from previous seen
-
-    Vector2D M_vel; //!< global velocity
-    int M_vel_count; //!< accuracy count
-
-    AngleDeg M_body; //!< global body angle
-    int M_body_count; //!< body angle accuracy
-    AngleDeg M_face; //!< global neck angle
-    int M_face_count; //!< face angle accuracy
-
     AngleDeg M_pointto_angle; //!< global pointing angle
     int M_pointto_count; //!< pointing angle accuracy
 
@@ -100,10 +80,11 @@ public:
 
     /*!
       \brief initialize member variables using observed info
+      \param side analyzed side info
       \param p analyzed seen player info
     */
-    explicit
-    PlayerObject( const Localization::PlayerT & p );
+    PlayerObject( const SideID side,
+                  const Localization::PlayerT & p );
 
     /*!
       \brief destructor. nothing to do
@@ -124,43 +105,6 @@ public:
 
     // ------------------------------------------
 
-    /*!
-      \brief get player's uniform number
-      \return uniform number. if unknown player, returned -1
-    */
-    int unum() const
-      {
-          return M_unum;
-      }
-
-    /*!
-      \brief get goalie flag
-      \return true if this player is goalie
-    */
-    bool goalie() const
-      {
-          return M_goalie;
-      }
-
-    /*!
-      \brief get the player type id
-      \return player type id
-     */
-    HeteroID type() const
-      {
-          return M_type;
-      }
-
-    /*!
-      \brief get the player type as a pointer.
-      \return player type pointer variable
-     */
-    const
-    PlayerType * playerTypePtr() const
-      {
-          return M_player_type_ptr;
-      }
-
     /*!
       \brief get distance from self
       \return distance value from self
@@ -181,16 +125,6 @@ public:
           return M_angle_from_self;
       }
 
-    /*!
-      \brief get distance from ball
-      \return distance value from ball
-    */
-    const
-    double & distFromBall() const
-      {
-          return M_dist_from_ball;
-      }
-
     /*!
       \brief check if this player is ghost object or not
       \return true if this player may be ghost object
@@ -209,25 +143,6 @@ public:
           return M_ghost_count;
       }
 
-    /*!
-      \brief get global position
-      \return const reference to the point object
-    */
-    const
-    Vector2D & pos() const
-      {
-          return M_pos;
-      }
-
-    /*!
-      \brief get global position accuracy
-      \return count from last observation
-    */
-    int posCount() const
-      {
-          return M_pos_count;
-      }
-
     /*!
       \brief get position relative to self position
       \return const reference to the point object
@@ -238,82 +153,6 @@ public:
           return M_rpos;
       }
 
-    /*!
-      \brief get the last seen position
-      \return const reference to the point object
-     */
-    const
-    Vector2D & prevPos() const
-      {
-          return M_prev_pos;
-      }
-
-    /*!
-      \brief get the number of cycles from previous seen
-      \return count from previous seen
-    */
-    int prevPosCount() const
-      {
-          return M_prev_pos_count;
-      }
-
-    /*!
-      \brief get velocity
-      \return const reference to the vector object
-    */
-    const
-    Vector2D & vel() const
-      {
-          return M_vel;
-      }
-
-    /*!
-      \brief get velocity accuracy
-      \return count from last observation
-    */
-    int velCount() const
-      {
-          return M_vel_count;
-      }
-
-    /*!
-      \brief get global body angle
-      \return const reference to the angle object
-    */
-    const
-    AngleDeg & body() const
-      {
-          return M_body; // global body angle
-      }
-
-    /*!
-      \brief get global body angle accuracy
-      \return count from last observation
-    */
-    int bodyCount() const
-      {
-          return M_body_count;
-      }
-
-    /*!
-      \brief get global neck angle
-      \return const reference to the angle object
-    */
-    const
-    AngleDeg & face() const
-      {
-          return M_face; // global neck angle
-      }
-
-    /*!
-      \brief get global neck angle accuracy
-      \return count from last observation
-    */
-    int faceCount() const
-      {
-          return M_face_count;
-      }
-
     /*!
       \brief get global pointing angle
       \return const reference to the angle object
@@ -401,18 +240,6 @@ public:
      */
     bool isKickable( const double & buf = 0.05 ) const;
 
-    // ------------------------------------------
-    /*!
-      \brief template method. check if player is in the region
-      \param region template resion. REGION must have method contains()
-      \return true if region contains player position
-    */
-    template < typename REGION >
-    bool isWithin( const REGION & region ) const
-      {
-          return region.contains( this->pos() );
-      }
-
     // ------------------------------------------
     /*!
       \brief update status only with intenal info
@@ -429,9 +256,11 @@ public:
 
     /*!
       \brief update status using localized player info
+      \param side analyzed side info
       \param p localized player info
     */
-    void updateBySee( const Localization::PlayerT & p );
+    void updateBySee( const SideID side,
+                      const Localization::PlayerT & p );
 
     /*!
       \brief update status using fullstate info
@@ -464,12 +293,6 @@ public:
                        const Vector2D & heard_pos,
                        const AngleDeg & heard_body );
 
-    /**!
-      \brief update player type id
-      \param id new id
-     */
-    void setPlayerType( const HeteroID type );
-
     /*!
       \brief update status related to other objects
       \param self self position
