@@ -34,12 +34,14 @@
 #endif
 
 #include "see_state.h"
-#include "logger.h"
 
+#include <rcsc/common/logger.h>
 #include <rcsc/common/server_param.h>
 
 namespace rcsc {
 
+bool SeeState::S_synch_see_mode = false;
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -67,16 +69,16 @@ SeeState::Timing
 SeeState::getNextTiming( const ViewWidth & vw,
                          const ViewQuality & vq ) const
 {
+    if ( S_synch_see_mode )
+    {
+        return TIME_SYNC;
+    }
+
     if ( vq.type() == ViewQuality::LOW )
     {
         return TIME_NOSYNCH;
     }
 
-    if ( vw.type() == ViewWidth::SYNC )
-    {
-        return TIME_SYNC;
-    }
-
     Timing timing = TIME_NOSYNCH;
 
     switch ( lastTiming() ) {
@@ -135,6 +137,44 @@ SeeState::getNextTiming( const ViewWidth & vw,
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+SeeState::updateBySenseBody( const GameTime & sense_time,
+                             const ViewWidth & vw,
+                             const ViewQuality & vq )
+{
+    setNewCycle( sense_time );
+
+    if ( M_view_width != vw )
+    {
+        dlog.addText( Logger::SYSTEM,
+                      "SeeState. updateBySenseBody. view_width does not match. old=%d sense=%d",
+                      M_view_width.type(), vw.type() );
+        std::cerr << sense_time
+                  << " view width does not match . old=" << M_view_width.type()
+                  << " sense=" << vw.type()
+                  << std::endl;
+
+        M_view_width = vw;
+    }
+
+    if ( M_view_quality != vq )
+    {
+        dlog.addText( Logger::SYSTEM,
+                      "SeeState. updateBySenseBody. view_quality does not match. old=%d sense=%d",
+                      M_view_quality.type(), vq.type() );
+        std::cerr << sense_time
+                  << " view quality does not match. old=" << M_view_quality.type()
+                  << " sense=" << vq.type()
+                  << std::endl;
+
+        M_view_quality = vq;
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 SeeState::updateBySee( const GameTime & see_time,
@@ -174,7 +214,15 @@ SeeState::updateBySee( const GameTime & see_time,
         return;
     }
 
+    //
     // see timing is synchronized.
+    //
+
+    if ( M_cycles_till_next_see > 0 )
+    {
+        M_cycles_till_next_see = 0;
+        setViewMode( vw, vq );
+    }
 
     // update current see arrival timing.
     Timing new_timing = getNextTiming( vw, vq );
@@ -242,10 +290,16 @@ SeeState::setLastSeeTiming( const Timing last_timing )
 bool
 SeeState::isSynch() const
 {
+    if ( S_synch_see_mode )
+    {
+        return true;
+    }
+
     if ( M_synch_type == SYNCH_SYNC )
     {
         return true;
     }
+
     return ( M_last_timing == TIME_0_00
              || M_last_timing == TIME_50_0
              || M_last_timing == TIME_22_5 );
@@ -297,6 +351,11 @@ SeeState::canChangeViewTo( const ViewWidth & next_width,
         return false;
     }
 
+    if ( S_synch_see_mode )
+    {
+        return true;
+    }
+
     if ( next_width.type() == ViewWidth::NARROW )
     {
         if ( lastTiming() == TIME_0_00 )
@@ -324,11 +383,6 @@ SeeState::canChangeViewTo( const ViewWidth & next_width,
         return true;
     }
 
-    if ( next_width.type() == ViewWidth::SYNC )
-    {
-        return true;
-    }
-
     std::cerr << __FILE__ << ':' << __LINE__
               << " unexpected reach..." << std::endl;
     return true;
@@ -359,6 +413,37 @@ SeeState::setViewMode( const ViewWidth & new_width,
         return;
     }
 
+    M_view_width = new_width;
+    M_view_quality = new_quality;
+
+    if ( S_synch_see_mode )
+    {
+        switch ( new_width.type() ) {
+        case ViewWidth::WIDE:
+            M_cycles_till_next_see = 3;
+            M_synch_type = SYNCH_WIDE;
+            dlog.addText( Logger::SYSTEM,
+                          "SeeState. setViewMode. synch wide: cycle = 3" );
+            break;
+        case ViewWidth::NORMAL:
+            M_cycles_till_next_see = 2;
+            M_synch_type = SYNCH_NORMAL;
+            dlog.addText( Logger::SYSTEM,
+                          "SeeState. setViewMode. synch normal: cycle = 2" );
+            break;
+        case ViewWidth::NARROW:
+            M_cycles_till_next_see = 1;
+            M_synch_type = SYNCH_NARROW;
+            dlog.addText( Logger::SYSTEM,
+                          "SeeState. setViewMode. synch narrow: cycle = 1" );
+            break;
+        default:
+            break;
+        }
+
+        return;
+    }
+
     // case 1
     if ( lastTiming() == TIME_0_00 )
     {
