@@ -40,8 +40,8 @@
 #include "debug_client.h"
 #include "body_sensor.h"
 #include "fullstate_sensor.h"
-#include "logger.h"
 
+#include <rcsc/common/logger.h>
 #include <rcsc/common/server_param.h>
 #include <rcsc/soccer_math.h>
 
@@ -60,22 +60,14 @@ int SelfObject::S_face_count_thr = 5;
 
 */
 SelfObject::SelfObject()
-    : M_time( -1, 0 )
+    : AbstractPlayerObject()
+    , M_time( -1, 0 )
     , M_sense_body_time( -1, 0 )
-    , M_unum( 0 )
-    , M_goalie( false )
-    , M_pos( Vector2D::INVALIDATED )
     , M_pos_error( 0.0, 0.0 )
-    , M_pos_count( 1000 )
     , M_pos_prev( Vector2D::INVALIDATED )
-    , M_vel( 0.0, 0.0 )
     , M_vel_error( 0.0, 0.0 )
-    , M_vel_count( 1000 )
-    , M_body( 0.0 )
     , M_neck( 0.0 )
-    , M_face( 0.0 )
     , M_face_error( 0.0 )
-    , M_face_count( 1000 )
     , M_view_width( ViewWidth::NORMAL )
     , M_view_quality( ViewQuality::HIGH )
     , M_last_catch_time( 0, 0 )
@@ -87,11 +79,21 @@ SelfObject::SelfObject()
     , M_attentionto_unum( 0 )
     , M_last_move( 0.0, 0.0 )
     , M_collision_estimated( false )
+    , M_collides_with_none( false )
+    , M_collides_with_ball( false )
+    , M_collides_with_player( false )
+    , M_collides_with_post( false )
     , M_kickable( false )
     , M_kick_rate( 0.0 )
     , M_tackle_probability( 0.0 )
 {
+    M_type = Hetero_Default;
+    M_player_type = PlayerTypeSet::i().get( Hetero_Default );
 
+    for ( int i = 0; i < 4; ++i )
+    {
+        M_last_moves[i].assign( 0.0, 0.0 );
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -113,9 +115,11 @@ SelfObject::set_count_thr( const int pos_thr,
 
 */
 void
-SelfObject::init( const int unum,
+SelfObject::init( const SideID side,
+                  const int unum,
                   const bool goalie )
 {
+    M_side = side;
     M_unum = unum;
     M_goalie = goalie;
 }
@@ -303,9 +307,8 @@ SelfObject::update( const ActionEffector & act,
         M_vel_error.add( velrnd, velrnd ); // add new vel rand
 
         dlog.addText( Logger::WORLD,
-                      "%s: update. pos=(%.2f, %.2f) pos_err(%.3f, %.3f) "
-                      "vel=(%.2f, %.2f) vel_err=(%.3f, %.3f)"
-                      ,__FILE__,
+                      __FILE__": update. pos=(%.2f, %.2f) pos_err(%.3f, %.3f) "
+                      "vel=(%.2f, %.2f) vel_err=(%.3f, %.3f)",
                       M_pos.x, M_pos.y, M_pos_error.x, M_pos_error.y,
                       M_vel.x, M_vel.y, M_vel_error.x, M_vel_error.y );
     }
@@ -321,7 +324,9 @@ SelfObject::update( const ActionEffector & act,
     /////////////////////////////////////////////////////////////////
     // update accuracy count
     ++M_pos_count;
+    ++M_seen_pos_count;
     ++M_vel_count;
+    ++M_body_count;
     ++M_face_count;
 
     // update action effect count
@@ -331,10 +336,19 @@ SelfObject::update( const ActionEffector & act,
     /////////////////////////////////////////////////////////////////
     // last move vector
     M_last_move = M_vel / this->playerType().playerDecay();
+    for ( int i = 2; i > 0; --i )
+    {
+        M_last_moves[i] = M_last_moves[i-1];
+    }
+    M_last_moves[0] = M_last_move;
 
     /////////////////////////////////////////////////////////////////
     // reset collision state
     M_collision_estimated = false;
+    M_collides_with_none = false;
+    M_collides_with_ball = false;
+    M_collides_with_player = false;
+    M_collides_with_post = false;
 }
 
 /*-------------------------------------------------------------------*/
@@ -443,7 +457,7 @@ SelfObject::updateAfterSense( const BodySensor & sense,
 
     ////////////////////////////////////////////////////
     // stamina
-    M_stamina.updateAfterSense( M_player_type,
+    M_stamina.updateAfterSense( playerType(),
                                 sense.stamina(), sense.effort(),
                                 current );
 
@@ -454,6 +468,45 @@ SelfObject::updateAfterSense( const BodySensor & sense,
         M_neck = sense.neckDir();
     }
 
+    //
+    // collision
+    //
+    if ( sense.noneCollided()
+         || sense.ballCollided()
+         || sense.playerCollided()
+         || sense.postCollided() )
+    {
+        M_collision_estimated = false; // !!! important !!!
+
+        if ( sense.noneCollided() )
+        {
+            M_collides_with_none = true;
+            dlog.addText( Logger::WORLD,
+                          __FILE__": self.updateAfterSense. no collision" );
+        }
+
+        if ( sense.ballCollided() )
+        {
+            M_collides_with_ball = true;
+            dlog.addText( Logger::WORLD,
+                          __FILE__": self.updateAfterSense. collided with ball" );
+        }
+
+        if ( sense.playerCollided() )
+        {
+            M_collides_with_player = true;
+            dlog.addText( Logger::WORLD,
+                          __FILE__": self.updateAfterSense. collided with player" );
+        }
+
+        if ( sense.postCollided() )
+        {
+            M_collides_with_post = true;
+            dlog.addText( Logger::WORLD,
+                          __FILE__": self.updateAfterSense. collided with post" );
+        }
+    }
+
     ////////////////////////////////////////////////////
     // face & vel
     if ( faceValid() )
@@ -500,23 +553,22 @@ SelfObject::updateAfterSense( const BodySensor & sense,
         M_vel_error.assign( ( maxcos - mincos ) * ( sense.speedMag() + 0.005 ),
                             ( maxsin - minsin ) * ( sense.speedMag() + 0.005 ) );
         dlog.addText( Logger::WORLD,
-                      "%s: updateAfterSense. vel=(%.2f %.2f) vel_err=(%.3f, %.3f)  faceErr = %.3f"
-                      ,__FILE__,
+                      __FILE__": updateAfterSense. vel=(%.2f %.2f) vel_err=(%.3f, %.3f)  faceErr = %.3f",
                       M_vel.x, M_vel.y,
                       M_vel_error.x, M_vel_error.y,
                       faceError() );
 
         ////////////////////////////////////////////////////
         // collision check
-        if ( velValid()
+        if ( ! hasSensedCollision()
+             && velValid()
              && sense.speedMag() < ( playerType().realSpeedMax()
                                      * playerType().playerDecay()
                                      * 0.11 )
              )
         {
             dlog.addText( Logger::WORLD,
-                          "%s: updateAfterSense. check collisiion. estimate_vel=(%.2f %.2f)"
-                          ,__FILE__,
+                          __FILE__": updateAfterSense. check collisiion. estimate_vel=(%.2f %.2f)",
                           estimate_vel.x, estimate_vel.y );
             // decay is very small
             //   default decay = 0.4
@@ -532,8 +584,7 @@ SelfObject::updateAfterSense( const BodySensor & sense,
             {
                 M_collision_estimated = true;
                 dlog.addText( Logger::WORLD,
-                              "%s: self.updateAfterSense. *COLLISION* estimated"
-                              ,__FILE__ );
+                              __FILE__": self.updateAfterSense. *COLLISION* estimated" );
             }
         }
 
@@ -543,13 +594,18 @@ SelfObject::updateAfterSense( const BodySensor & sense,
 
         //////////////////////////////////////////////////////
         // pointto
-        if ( posValid() )
+        if ( sense.armExpires() != 0
+             && posValid() )
         {
             M_pointto_pos
                 = M_pos
                 + Vector2D::polar2vector( sense.pointtoDist(),
                                           face() + sense.pointtoDir() );
         }
+        else
+        {
+            M_pointto_pos.invalidate();
+        }
 
         ////////////////////////////////////////////////////
         // last move
@@ -564,8 +620,19 @@ SelfObject::updateAfterSense( const BodySensor & sense,
         }
         else
         {
+            dlog.addText( Logger::WORLD,
+                          __FILE__": updateAfterSense. XXX reset last move" );
             M_last_move.invalidate();
         }
+        M_last_moves[0] = M_last_move;
+
+        if ( collisionEstimated()
+             || collidesWithBall() )
+        {
+            dlog.addText( Logger::WORLD,
+                          __FILE__": updateAfterSense. collision. reset last move" );
+            M_last_moves[0].invalidate();
+        }
     }
 
     // attentionto
@@ -576,8 +643,7 @@ SelfObject::updateAfterSense( const BodySensor & sense,
     if ( M_tackle_expires > 0 )
     {
         dlog.addText( Logger::WORLD,
-                      "%s: updateAfterSense internal_tackle_exp=%d, sense_body=%d"
-                      ,__FILE__,
+                      __FILE__": updateAfterSense internal_tackle_exp=%d, sense_body=%d",
                       M_tackle_expires, sense.tackleExpires() );
     }
 
@@ -587,8 +653,7 @@ SelfObject::updateAfterSense( const BodySensor & sense,
                   << "self : updateAfterSense. internal tackle expire has invalie value."
                   << std::endl;
         dlog.addText( Logger::WORLD,
-                      "%s: updateAfterSense internal_tackle_exp is not match to sense_body"
-                      ,__FILE__ );
+                      __FILE__": updateAfterSense internal_tackle_exp is not match to sense_body" );
     }
 
     M_tackle_expires = sense.tackleExpires();
@@ -610,11 +675,15 @@ SelfObject::updateAfterFullstate( const FullstateSensor::PlayerT & my_state,
     M_pos_error.assign( 0.0, 0.0 );
     M_pos_count = 0;
 
+    M_seen_pos = my_state.pos_;
+    M_seen_pos_count = 0;
+
     M_vel = my_state.vel_;
     M_vel_error.assign( 0.0, 0.0 );
     M_vel_count = 0;
 
     M_body = my_state.body_;
+    M_body_count = 0;
     M_neck = my_state.neck_;
     M_face = M_body + M_neck;
     M_face_error = 0.0;
@@ -659,8 +728,7 @@ SelfObject::updatePos( const Vector2D & pos,
                 + ( M_pos.x - pos.x ) * ( pos_err.x / ( M_pos_error.x + pos_err.x ) );
             new_error.x = ( M_pos_error.x + pos_err.x ) * 0.5;
             dlog.addText( Logger::WORLD,
-                          "%s: updatePos. X modified [%f] < [%f]"
-                          ,__FILE__,
+                          __FILE__": updatePos. X modified [%f] < [%f]",
                           M_pos_error.x, pos_err.x );
         }
         if  ( M_pos_error.y < pos_err.y )
@@ -670,17 +738,15 @@ SelfObject::updatePos( const Vector2D & pos,
                 + ( M_pos.y - pos.y ) * ( pos_err.y / ( M_pos_error.y + pos_err.y ) );
             new_error.y = ( M_pos_error.y + pos_err.y ) * 0.5;
             dlog.addText( Logger::WORLD,
-                          "%s: updatePos. Y modified [%f] < [%f]"
-                          ,__FILE__,
+                          __FILE__": updatePos. Y modified [%f] < [%f]",
                           M_pos_error.y, pos_err.y );
         }
         M_pos = new_pos;
         M_pos_error = new_error;
 
         dlog.addText( Logger::WORLD,
-                      "%s: updatePos. seen_pos=(%.3f, %.3f) average_pos=(%.3f, %.3f) "
-                      " error=(%.4f, %.4f)"
-                      ,__FILE__,
+                      __FILE__": updatePos. seen_pos=(%.3f, %.3f) average_pos=(%.3f, %.3f) "
+                      " error=(%.4f, %.4f)",
                       pos.x, pos.y, new_pos.x, new_pos.y,
                       M_pos_error.x, M_pos_error.y );
     }
@@ -691,8 +757,12 @@ SelfObject::updatePos( const Vector2D & pos,
     }
 
     M_pos_count = 0;
+    M_seen_pos = M_pos;
+    M_seen_pos_count = 0;
+
     M_face = face;
     M_body = face - M_neck.degree();
+    M_body_count = 0;
     M_face_error = face_err;
     M_face_count = 0;
 }
@@ -700,6 +770,18 @@ SelfObject::updatePos( const Vector2D & pos,
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+SelfObject::updateByCollision( const Vector2D & pos,
+                               const Vector2D & pos_error )
+{
+    M_pos = pos;
+    M_pos_error = pos_error;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 SelfObject::updateAngle( const double & face,
@@ -709,6 +791,7 @@ SelfObject::updateAngle( const double & face,
     M_time = current;
     M_face = face;
     M_body = face - M_neck.degree();
+    M_body_count = 0;
     M_face_error = face_err;
     M_face_count = 0;
 }
@@ -769,11 +852,11 @@ SelfObject::updateVelDirAfterSee( const BodySensor & sense,
 
         // set last move
         M_last_move = M_vel / this->playerType().playerDecay();
+        M_last_moves[0] = M_last_move;
 
         dlog.addText( Logger::WORLD,
-                      "%s: updateVelDirAfterSee. face_error=%.2f, sensed_dir_error=%.2f."
-                      " vel=(%f %f) vel_err=(%f %f) last_move=(%.4f %.4f)"
-                      ,__FILE__,
+                      __FILE__": updateVelDirAfterSee. face_error=%.2f, sensed_dir_error=%.2f."
+                      " vel=(%f %f) vel_err=(%f %f) last_move=(%.4f %.4f)",
                       faceError(), sensed_speed_dir_error,
                       M_vel.x, M_vel.y,
                       M_vel_error.x, M_vel_error.y,
@@ -797,6 +880,8 @@ SelfObject::updateBallInfo( const BallObject & ball )
         return;
     }
 
+    M_dist_from_ball = ball.distFromSelf();
+
     //------------------------------------------------------
     // check kickable
 
@@ -845,8 +930,7 @@ SelfObject::updateBallInfo( const BallObject & ball )
     }
 
     dlog.addText( Logger::WORLD,
-                  "%s: updateBallInfo. ball_dist=%.3f.  my_kickable=%.3f.  tackle_prob=%.3f"
-                  ,__FILE__,
+                  __FILE__": updateBallInfo. ball_dist=%.3f.  my_kickable=%.3f.  tackle_prob=%.3f",
                   ball.distFromSelf(), kickableArea(), M_tackle_probability );
 }
 
@@ -868,14 +952,13 @@ SelfObject::getSafetyDashPower( const double & dash_power ) const
                   << " input dash power = " << dash_power
                   << std::endl;
         dlog.addText( Logger::ACTION,
-                      "%d:%d:  required stamina should be positive now"
-                      ,__FILE__, __LINE__ );
+                      __FILE__": getSafetyDashPower. required stamina should be positive now" );
     }
 
     // check rest stamina
 
     const double safety_stamina
-        = stamina() - ServerParam::i().effortDecThrValue() - 3.0;
+        = stamina() - ServerParam::i().recoverDecThrValue() - 3.0;
 
     double available_stamina = std::max( 0.0, safety_stamina );
 
@@ -892,8 +975,7 @@ SelfObject::getSafetyDashPower( const double & dash_power ) const
     }
 
     dlog.addText( Logger::ACTION,
-                  "%s:%d: getSafetyDashPower. cur_stamina=%.1f, required=%.1f, result=%.2f"
-                  ,__FILE__, __LINE__,
+                  __FILE__": getSafetyDashPower. cur_stamina=%.1f, required=%.1f, result=%.2f",
                   stamina(), required_stamina, result_power );
 
 
