@@ -32,6 +32,7 @@
 #ifndef RCSC_PLAYER_SELF_OBJECT_H
 #define RCSC_PLAYER_SELF_OBJECT_H
 
+#include <rcsc/player/abstract_player_object.h>
 #include <rcsc/player/view_mode.h>
 #include <rcsc/player/stamina_model.h>
 #include <rcsc/player/fullstate_sensor.h>
@@ -48,7 +49,8 @@ class ActionEffector;
 class BallObject;
 
 //! self object class
-class SelfObject {
+class SelfObject
+    : public AbstractPlayerObject {
 private:
     //! validation count threshold value for M_pos
     static int S_pos_count_thr;
@@ -62,32 +64,15 @@ private:
     //! time updated by sense_body
     GameTime M_sense_body_time;
 
-    // team info
-
-    //! uniform number
-    int  M_unum;
-    //! goalie flag
-    bool M_goalie;
-
-    // parameter
-    //! current player type paramter
-    PlayerType M_player_type;
 
     // status on field
-    Vector2D M_pos; //!< estimated global position
     Vector2D M_pos_error; //!< estimated error of global position
-    int M_pos_count; //!< count from last self localization
     Vector2D M_pos_prev; //!< global position at previous cycle
 
-    Vector2D M_vel; //!< estimated velocity
     Vector2D M_vel_error; //!< estimated error of velocity
-    int M_vel_count; //!< count from last velocity observation
 
-    AngleDeg M_body; //!< global body angle
     AngleDeg M_neck; //!< relative neck angle
-    AngleDeg M_face; //!< global neck angle
     double M_face_error; //!< estimated error of angle info
-    int M_face_count; //!< count from last angle observation
 
     ViewWidth M_view_width; //!< current view width type
     ViewQuality M_view_quality; //!< current view quality type
@@ -109,8 +94,14 @@ private:
 
     //! last move vector (transformation from previous to current)
     Vector2D M_last_move;
+    //! last 3 move vectors
+    Vector2D  M_last_moves[4];
 
     bool M_collision_estimated; //!< collision estimation result
+    bool M_collides_with_none; //!< true if agent does not collide
+    bool M_collides_with_ball; //!< true if collided with ball
+    bool M_collides_with_player; //!< true if collided with player
+    bool M_collides_with_post; //!< true if collided with post
 
     // status values for action
     bool M_kickable; //!< ball kickable flag
@@ -121,6 +112,7 @@ private:
     SelfObject( const SelfObject & self );
     //! not used
     SelfObject & operator=( const SelfObject & self );
+
 public:
 
     /*!
@@ -146,49 +138,34 @@ public:
 
     /*!
       \brief set uniform number and goalie flag.
+      \param side side id returned from server
       \param unum uniform number returned from server
       \param goalie true if agent connected as goalie.
 
       This method is called just after received init replay message.
     */
-    void init( const int unum,
+    void init( const SideID side,
+               const int unum,
                const bool goalie );
 
-    /*!
-      \brief get uniform number
-      \return uniform number
-    */
-    int unum() const
+   /*!
+      \brief check if this player is self or not
+      \return true if this player is self
+     */
+    virtual
+    bool isSelf() const
       {
-          return M_unum;
-      }
-
-    /*!
-      \brief get goalie flag
-      \return true if goalie
-    */
-    bool goalie() const
-      {
-          return M_goalie;
-      }
-
-    /*!
-      \brief set heterogenious player type
-      \param player_type player type parameter object
-    */
-    void setPlayerType( const PlayerType & player_type )
-      {
-          M_player_type = player_type;
+          return true;
       }
 
     /*!
       \brief get player type parameter
-      \return const referenct to the player type object
+      \return const reference to the player type object
     */
     const
     PlayerType & playerType() const
       {
-          return M_player_type;
+          return *M_player_type;
       }
 
     /*!
@@ -209,19 +186,10 @@ public:
     */
     bool faceValid() const;
 
-    /*!
-      \brief get agent's estimated global position
-      \return const reference to the position object
-    */
-    const
-    Vector2D & pos() const
-      {
-          return M_pos;
-      }
 
     /*!
       \brief get estimated error of agent's global position
-      \return const referenct to the error vector
+      \return const reference to the error vector
     */
     const
     Vector2D & posError() const
@@ -229,18 +197,9 @@ public:
           return M_pos_error;
       }
 
-    /*!
-      \brief get position accuracy
-      \return count from last self localization
-    */
-    int posCount() const
-      {
-          return M_pos_count;
-      }
-
     /*!
       \brief get global position at previous cycle
-      \return const referenct to the position object
+      \return const reference to the position object
     */
     const
     Vector2D & posPrev() const
@@ -248,16 +207,6 @@ public:
           return M_pos_prev;
       }
 
-    /*!
-      \brief get estimated velocity
-      \return const reference to the velocity object
-    */
-    const
-    Vector2D & vel() const
-      {
-          return M_vel;
-      }
-
     /*!
       \brief get estimated speed
       \return scalar value of current velocity
@@ -269,7 +218,7 @@ public:
 
     /*!
       \brief get estimated error of velocity
-      \return const referenct to the error vector object
+      \return const reference to the error vector object
     */
     const
     Vector2D & velError() const
@@ -277,29 +226,9 @@ public:
           return M_vel_error;
       }
 
-    /*!
-      \brief get velocity accuracy
-      \return count from last observation
-    */
-    const
-    int velCount() const
-      {
-          return M_vel_count;
-      }
-
-    /*!
-      \brief get estimated body angle
-      \return const referenct to the angle object
-    */
-    const
-    AngleDeg & body() const
-      {
-          return M_body;
-      }
-
     /*!
       \brief get estimated neck angle relative to body angle
-      \return const referenct to the angle object
+      \return const reference to the angle object
     */
     const
     AngleDeg & neck() const
@@ -307,16 +236,6 @@ public:
           return M_neck;
       }
 
-    /*!
-      \brief get estimated face angle (global neck angle)
-      \return const referenct to the angle object
-    */
-    const
-    AngleDeg & face() const
-      {
-          return M_face;
-      }
-
     /*!
       \brief get estimated error about angle
       \return error value about angle
@@ -327,15 +246,6 @@ public:
           return M_face_error;
       }
 
-    /*!
-      \brief get angle accuracy
-      \return count from last observation
-    */
-    int faceCount() const
-      {
-          return M_face_count;
-      }
-
     /*!
       \brief get current view width
       \return const reference to the view width object
@@ -465,11 +375,59 @@ public:
       \brief get estimated collistion status
       \return true collision is estimated
     */
-    bool collision() const
+    bool collisionEstimated() const
       {
           return M_collision_estimated;
       }
 
+    /*!
+      \brief get a flag whether agent has a certain collistion info
+      \return true if agent has a certain collision info
+    */
+    bool hasSensedCollision() const
+      {
+          return M_collides_with_none
+              || M_collides_with_ball
+              || M_collides_with_player
+              || M_collides_with_post;
+      }
+
+    /*!
+      \brief get a certain collistion status with no objects
+      \return true if no collision
+    */
+    bool collidesWithNone() const
+      {
+          return M_collides_with_none;
+      }
+
+    /*!
+      \brief get a certain collistion status with ball
+      \return true collision occured
+    */
+    bool collidesWithBall() const
+      {
+          return M_collides_with_ball;
+      }
+
+    /*!
+      \brief get a certain collistion status with players
+      \return true collision occured
+    */
+    bool collidesWithPlayer() const
+      {
+          return M_collides_with_ball;
+      }
+
+    /*!
+      \brief get a certain collistion status with posts
+      \return true collision occured
+    */
+    bool collidesWithPost() const
+      {
+          return M_collides_with_post;
+      }
+
     /*!
       \brief get last move vector (transformation from previous to current)
       \return const reference to the vector object
@@ -480,6 +438,17 @@ public:
           return M_last_move;
       }
 
+    /*!
+      \brief get last 3 move vectors
+      \return const reference to the vector object
+    */
+    const
+    Vector2D & lastMove( const int i ) const
+      {
+          if ( i < 0 || 2 < i ) return M_last_moves[3];
+          return M_last_moves[i];
+      }
+
     /*!
       \brief get ball kickable status
       \return true if ball kickable
@@ -583,6 +552,14 @@ public:
                     const double & face_err,
                     const GameTime & current );
 
+    /*!
+      \brief apply collision effect
+      \param pos new global position
+      \param pos_error estimated maximum position error
+     */
+    void updateByCollision( const Vector2D & pos,
+                            const Vector2D & pos_error );
+
     /*!
       \brief update status using see info. but only angle.
       \param face estimated global neck angle
