@@ -35,7 +35,9 @@
 
 #include "view_mode.h"
 
-#include <rcsc/param/server_param.h>
+#include "see_state.h"
+
+#include <rcsc/common/server_param.h>
 
 #include <iostream>
 #include <cstring>
@@ -47,24 +49,41 @@ namespace rcsc {
 
 */
 double
-ViewWidth::getWidth() const
+ViewWidth::width() const
 {
-    switch ( this->type() ) {
-    case ViewWidth::NARROW:
-        return ServerParam::i().visibleAngle() * 0.5;
-    case ViewWidth::NORMAL:
-        return ServerParam::i().visibleAngle();
-    case ViewWidth::WIDE:
-        return ServerParam::i().visibleAngle() * 2.0;
-    case ViewWidth::SYNC:
-        return ServerParam::i().visibleAngle() *
-            ( (double)ServerParam::i().simulatorStep()
-              / (double)ServerParam::i().sendStep() );
-    default:
-        break;
+    if ( SeeState::synch_see_mode() )
+    {
+        double r
+            = (double)ServerParam::i().simulatorStep()
+            / (double)ServerParam::i().sendStep();
+        switch ( this->type() ) {
+        case ViewWidth::NARROW:
+            return ServerParam::i().visibleAngle() * r;
+        case ViewWidth::NORMAL:
+            return ServerParam::i().visibleAngle() * 2.0 * r;
+        case ViewWidth::WIDE:
+            return ServerParam::i().visibleAngle() * 3.0 * r;
+        default:
+            break;
+        }
     }
+    else
+    {
+        switch ( this->type() ) {
+        case ViewWidth::NARROW:
+            return ServerParam::i().visibleAngle() * 0.5;
+        case ViewWidth::NORMAL:
+            return ServerParam::i().visibleAngle();
+        case ViewWidth::WIDE:
+            return ServerParam::i().visibleAngle() * 2.0;
+        default:
+            break;
+        }
+    }
+
     std::cerr << __FILE__ << ':' << __LINE__
               << "Unknown View Width" << std::endl;
+
     return ServerParam::i().visibleAngle();
 }
 
@@ -82,8 +101,6 @@ ViewWidth::str() const
         return std::string( "normal" );
     case ViewWidth::WIDE:
         return std::string( "wide" );
-    case ViewWidth::SYNC:
-        return std::string( "sync" );
     default:
         break;
     }
@@ -111,10 +128,6 @@ ViewWidth::parse( const char * msg )
     {
         return ViewWidth::WIDE;
     }
-    else if ( ! std::strncmp( msg, "sync", 4 ) )
-    {
-        return ViewWidth::SYNC;
-    }
     else
     {
         std::cerr << __FILE__ << ':' << __LINE__
