@@ -35,7 +35,7 @@
 
 #include "visual_sensor.h"
 
-#include "logger.h"
+#include <rcsc/common/logger.h>
 
 #include <cstdlib>
 #include <cmath> // HUGE_VAL
