@@ -55,6 +55,7 @@ class BodySensor;
 class FullstateSensor;
 class InterceptTable;
 class Localization;
+class PlayerPredicate;
 class PlayerType;
 class PenaltyKickState;
 class VisualSensor;
@@ -114,7 +115,7 @@ private:
     //     or update_after_see
     //   pointers to each object
     //     updated in  void refreshPlayerMatrix();
-    //! sorted by distance fromo self
+    //! sorted by distance from self
     PlayerPtrCont M_teammates_from_self;
     //! sorted by distance from ball. include unknown players
     PlayerPtrCont M_opponents_from_self;
@@ -126,6 +127,13 @@ private:
     //! uniform number of opponent goalie
     int M_opponent_goalie_unum;
 
+    AbstractPlayerCont M_all_players; //!< all players including self
+    AbstractPlayerCont M_all_teammates; //!< all teammates including self
+    AbstractPlayerCont M_all_opponents; //!< all opponents
+
+    AbstractPlayerObject * M_known_teammates[12]; //!< unum known teammates (includes self)
+    AbstractPlayerObject * M_known_opponents[12]; //!< unum known opponents (excludes unknown player)
+
     //////////////////////////////////////////////////
     // strategic value
 
@@ -141,8 +149,8 @@ private:
     //////////////////////////////////////////////////
     // player type management
 
-    HeteroID M_teammate_types[11]; //!< teammate type reference
-    HeteroID M_opponent_types[11]; //!< opponent type flag
+    int M_teammate_types[11]; //!< teammate type reference
+    int M_opponent_types[11]; //!< opponent type flag
 
     //////////////////////////////////////////////////
     // visual info
@@ -184,7 +192,7 @@ public:
 
     /*!
       \brief get audio memory
-      \return co
+      \return const reference to the audio memory instance
      */
     const
     AudioMemory & audioMemory() const
@@ -220,7 +228,7 @@ public:
       \param id player type ID
     */
     void setTeammatePlayerType( const int unum,
-                                const HeteroID id );
+                                const int id );
 
     /*!
       \brief set opponent player type
@@ -228,7 +236,7 @@ public:
       \param id player type ID
     */
     void setOpponentPlayerType( const int unum,
-                                const HeteroID id );
+                                const int id );
 
     // update stuff
 private:
@@ -331,11 +339,33 @@ private:
     /*!
       \brief ball localization
       \param see analyzed see info
+      \param act action effector
       \param current current game time
     */
     void localizeBall( const VisualSensor & see,
+                       const ActionEffector & act,
                        const GameTime & current );
 
+    /*!
+      \brief estimate ball velocity using position difference
+      \param see analyzed see info
+      \param act action effector
+      \param current current game time
+      \param current seen relative pos
+      \param current seen relative pos error
+      \param vel reference to the velocity variable
+      \param vel_error reference to the velocity error variable
+      \param vel_count reference to the velocity count variable
+    */
+    void estimateBallVelByPosDiff( const VisualSensor & see,
+                                   const ActionEffector & act,
+                                   const GameTime & current,
+                                   const Vector2D & rpos,
+                                   const Vector2D & rpos_error,
+                                   Vector2D & vel,
+                                   Vector2D & vel_error,
+                                   int & vel_count );
+
     /*!
       \brief players localization
       \param see analyzed see info
@@ -345,13 +375,15 @@ private:
 
     /*!
       \brief check player that has team info
+      \param side seen side info
       \param player localized info
       \param seen_dist see info data
       \param old_known_players old team known players
       \param old_unknown_players previous unknown players
       \param new_known_players new team known players
     */
-    void checkTeamPlayer( const Localization::PlayerT & player,
+    void checkTeamPlayer( const SideID side,
+                          const Localization::PlayerT & player,
                           const double & seen_dist,
                           PlayerCont & old_known_players,
                           PlayerCont & old_unknown_players,
@@ -654,6 +686,36 @@ public:
           return M_opponent_goalie_unum;
       }
 
+    /*!
+      \brief get all players including self.
+      \return const rerefence to the AbstractPlayerObject pointer container..
+     */
+    const
+    AbstractPlayerCont & allPlayers() const
+      {
+          return M_all_players;
+      }
+
+    /*!
+      \brief get all teammates including self.
+      \return const rerefence to the AbstractPlayerObject pointer container..
+     */
+    const
+    AbstractPlayerCont & allTeammates() const
+      {
+          return M_all_teammates;
+      }
+
+    /*!
+      \brief get all opponents including self.
+      \return const rerefence to the AbstractPlayerObject pointer container..
+     */
+    const
+    AbstractPlayerCont & allOpponents() const
+      {
+          return M_all_opponents;
+      }
+
     //////////////////////////////////////////////////////////
 
     /*!
@@ -661,20 +723,26 @@ public:
       \param unum wanted player's uniform number
       \return const pointer to the PlayerObject instance or NULL
     */
+//     const
+//     PlayerObject * getTeammate( const int unum ) const
+//       {
+//           const PlayerPtrCont::const_iterator end = teammatesFromSelf().end();
+//           for ( PlayerPtrCont::const_iterator it = teammatesFromSelf().begin();
+//                 it != end;
+//                 ++it )
+//           {
+//               if ( (*it)->unum() == unum )
+//               {
+//                   return *it;
+//               }
+//           }
+//           return static_cast< PlayerObject * >( 0 );
+//       }
     const
-    PlayerObject * getTeammate( const int unum ) const
+    AbstractPlayerObject * teammate( const int unum ) const
       {
-          const PlayerPtrCont::const_iterator end = teammatesFromSelf().end();
-          for ( PlayerPtrCont::const_iterator it = teammatesFromSelf().begin();
-                it != end;
-                ++it )
-          {
-              if ( (*it)->unum() == unum )
-              {
-                  return *it;
-              }
-          }
-          return static_cast< PlayerObject * >( 0 );
+          if ( unum <= 0 || 11 < unum ) return M_known_teammates[0];
+          return M_known_teammates[unum];
       }
 
     /*!
@@ -682,20 +750,26 @@ public:
       \param unum wanted player's uniform number
       \return const pointer to the PlayerObject instance or NULL
     */
+//     const
+//     PlayerObject * getOpponent( const int unum ) const
+//       {
+//           const PlayerPtrCont::const_iterator end = opponentsFromSelf().end();
+//           for ( PlayerPtrCont::const_iterator it = opponentsFromSelf().begin();
+//                 it != end;
+//                 ++it )
+//           {
+//               if ( (*it)->unum() == unum )
+//               {
+//                   return *it;
+//               }
+//           }
+//           return static_cast< PlayerObject * >( 0 );
+//       }
     const
-    PlayerObject * getOpponent( const int unum ) const
+    AbstractPlayerObject * opponent( const int unum ) const
       {
-          const PlayerPtrCont::const_iterator end = opponentsFromSelf().end();
-          for ( PlayerPtrCont::const_iterator it = opponentsFromSelf().begin();
-                it != end;
-                ++it )
-          {
-              if ( (*it)->unum() == unum )
-              {
-                  return *it;
-              }
-          }
-          return static_cast< PlayerObject * >( 0 );
+          if ( unum <= 0 || 11 < unum ) return M_known_opponents[0];
+          return M_known_opponents[unum];
       }
 
     /*!
@@ -703,7 +777,7 @@ public:
       count threshold
       \param first first iterator of PlayerObject pointer container
       \param last last iterator of PlayerObject pointer container
-      \param count_thr confidence count threshold
+      \param count_thr accuracy count threshold
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
     const
@@ -725,7 +799,7 @@ public:
 
     /*!
       \brief get teammate nearest to self with confidence count check
-      \param count_thr confidence count threshold
+      \param count_thr accuracy count threshold
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
     const
@@ -773,7 +847,7 @@ public:
 
     /*!
       \brief get teammate nearest to with with confidence count check
-      \param count_thr confidence count threshold
+      \param count_thr accuracy count threshold
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
     const
@@ -786,7 +860,7 @@ public:
 
     /*!
       \brief get opponent nearest to ball with confidence count check
-      \param count_thr confidence count threshold
+      \param count_thr accuracy count threshold
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
     const
@@ -873,11 +947,11 @@ public:
       \param unum target teammate uniform number
       \return player type Id. if unum is illegal, Default Id is returned.
     */
-    HeteroID teammateHeteroID( const int unum ) const
+    int teammateHeteroID( const int unum ) const
       {
           if ( unum < 1 || 11 < unum )
           {
-              std::cerr << "WorldModel::getTeammateHeteroID. Invalid unum "
+              std::cerr << "WorldModel::teammateHeteroID. Illegal unum "
                         << unum << std::endl;
               return Hetero_Default;
           }
@@ -897,12 +971,11 @@ public:
       \param unum target teammate uniform number
       \return player type Id. if unum is illegal, Unknown is returned.
     */
-    const
-    HeteroID opponentHeteroID( const int unum ) const
+    int opponentHeteroID( const int unum ) const
       {
           if ( unum < 1 || 11 < unum )
           {
-              std::cerr << "WorldModel::getOpponentHeteroID. Invalid unum "
+              std::cerr << "WorldModel::opponentHeteroID. Illegal unum "
                         << unum << std::endl;
               return Hetero_Unknown;
           }
@@ -952,7 +1025,25 @@ public:
                        int * sum_count,
                        int * ave_count ) const;
 
-    // some special interface
+    //
+    // interfaces to player objects
+    //
+
+    /*!
+      \brief get the new container of AbstractPlayer matched with the predicate.
+      \param predicate predicate object for the player condition matching. This have to be a dynamically allocated object.
+      \return container of AbstractPlayer pointer.
+     */
+    AbstractPlayerCont getPlayerCont( const PlayerPredicate * predicate ) const;
+
+    /*!
+      \brief get the new container of AbstractPlayer matched with the predicate.
+      \param cont reference to the result variable
+      \param predicate predicate object for the player condition matching. This have to be a dynamically allocated object.
+     */
+    void getPlayerCont( AbstractPlayerCont & cont,
+                        const PlayerPredicate * predicate ) const;
+
 
     /*!
       \brief get opponent goalie pointer
@@ -974,10 +1065,27 @@ public:
                                          const int count_thr,
                                          double * dist_to_point ) const;
 
+    /*!
+      \brief get teammate pointer nearest to the specified player
+      \param p pointer to the player
+      \param count_thr accuracy count threshold
+      \param dist_to_point variable pointer to store the distance
+      from retuned player to point
+      \return if found, pointer to player object, othewise NULL
+     */
+    const
+    PlayerObject * getTeammateNearestTo( const PlayerObject * p,
+                                         const int count_thr,
+                                         double * dist_to_point ) const
+      {
+          if ( ! p ) return static_cast< const PlayerObject * >( 0 );
+          return getTeammateNearestTo( p->pos(), count_thr, dist_to_point );
+      }
+
     /*!
       \brief get opponent pointer nearest to point
       \param point considered point
-      \param count_thr cinfidence count threshold
+      \param count_thr accuracy count threshold
       \param dist_to_point variable pointer to store the distance
       from retuned player to point
       \return if found pointer to player object, othewise NULL
@@ -987,6 +1095,24 @@ public:
                                          const int count_thr,
                                          double * dist_to_point ) const;
 
+
+    /*!
+      \brief get teammate pointer nearest to the specified player
+      \param p pointer to the player
+      \param count_thr accuracy count threshold
+      \param dist_to_point variable pointer to store the distance
+      from retuned player to point
+      \return if found, pointer to player object, othewise NULL
+     */
+    const
+    PlayerObject * getOpponentNearestTo( const PlayerObject * p,
+                                         const int count_thr,
+                                         double * dist_to_point ) const
+      {
+          if ( ! p ) return static_cast< const PlayerObject * >( 0 );
+          return getOpponentNearestTo( p->pos(), count_thr, dist_to_point );
+      }
+
     /*!
       \brief template utility. check if teammate exist in the specified region.
       \param region template parameter. region to be checked
