@@ -35,9 +35,15 @@
 
 #include "holder.h"
 
+#include "reader.h"
+#include "parser_v4.h"
+
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
+#ifdef HAVE_WINDOWS_H
+#include <windows.h>
+#endif
 
 #include <iostream>
 
@@ -110,5 +116,20 @@ Holder::addDispInfo2( const dispinfo_t2 & dinfo2 )
     return false;
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+bool
+Holder::addDisp3( const std::string & msg )
+{
+    setLogVersion( REC_VERSION_4 );
+
+    Reader reader( *this );
+    ParserV4 parser;
+
+    return parser.parseLine( 0, msg, reader );
+}
+
 } // end namespace
 } // end namespace
