@@ -36,13 +36,20 @@
 
 #include <boost/cstdint.hpp>
 
+#include <string>
+
 namespace rcsc {
 namespace rcg {
 
-//! type of the short value
+
+//! type of the 16bits integer value
 typedef boost::int16_t Int16;
-//! type of the long value
+//! type of the unsigned 16bits integer value
+typedef boost::uint16_t UInt16;
+//! type of the 32bits integer value
 typedef boost::int32_t Int32;
+//! type of the unsigned 32bits integer value
+typedef boost::int32_t UInt32;
 
 /*!
   \brief max length of color name string.
@@ -107,6 +114,7 @@ enum PlayerStatus {
 // game log format versin 1 & version 2
 
 const double SHOWINFO_SCALE = 16.0; //!< scaling parameter
+const float SHOWINFO_SCALEF = 16.0f;
 const int REC_OLD_VERSION = 1; //!< version number of rcg v1
 const int REC_VERSION_2 = 2; //!< recorded value of rcg v2
 
@@ -217,6 +225,8 @@ struct dispinfo_t {
 
 //! scaling variable
 const double SHOWINFO_SCALE2 = 65536.0;
+//! scaling variable
+const float SHOWINFO_SCALE2F = 65536.0f;
 //! version number
 const int REC_VERSION_3 = 3;
 
@@ -422,13 +432,14 @@ struct server_params_t {
     Int32 ka_length; //!< keep away region length
     Int32 ka_width;  //!< keep away region width
 
-    Int32 ball_stuck_area; //!< spare variable
+    Int32 ball_stuck_area; //!< automatic drop ball threshold while play_on
 
-    Int32 sparelong6; //!< spare variable
-    Int32 sparelong7; //!< spare variable
-    Int32 sparelong8; //!< spare variable
-    Int32 sparelong9; //!< spare variable
-    Int32 sparelong10; //!< spare variable
+    Int32 max_tackle_power; //!< allowed maximum tackle power
+    Int32 max_back_tackle_power; //!< allowed minimum tackle power
+
+    Int32 tackle_dist; //!< forward tackle area length
+    Int32 tackle_back_dist; //!< back tackle area length
+    Int32 tackle_width; //!< tackle area width
 
     Int16 start_goal_l; //!< initial left team score
     Int16 start_goal_r; //!< initial right team score
@@ -491,7 +502,8 @@ struct player_params_t {
     Int32 sparelong9; //!< spare variable
     Int32 sparelong10; //!< spare variable
 
-    Int16 spareshort1; //!< spare variable
+    Int16 allow_mult_default_type; //!< whether multiple default type is allowed
+
     Int16 spareshort2; //!< spare variable
     Int16 spareshort3; //!< spare variable
     Int16 spareshort4; //!< spare variable
@@ -520,8 +532,238 @@ struct dispinfo_t2 {
     } body; //!< union variable
 };
 
+//
+// Data structures for the text based monitor protocl
+//
+
+/*!
+  \struct BallT
+  \brief generic ball data for display information
+ */
+struct BallT {
+    float x_; //!< ball position x
+    float y_; //!< ball position y
+    float vx_; //!< ball velocity x
+    float vy_; //!< ball velocity y
+
+    /*!
+      \brief initialize all variables by 0
+     */
+    BallT()
+        : x_( 0.0 )
+        , y_( 0.0 )
+        , vx_( 0.0 )
+        , vy_( 0.0 )
+      { }
+};
+
+/*!
+  \struct PlayerT
+  \brief generic player data for display information
+ */
+struct PlayerT {
+    char side_; //!< player's side. 'l', 'r' or 'n'
+    Int16 unum_; //!< player's uniform number. 0 means disabled player.
+    Int16 type_; //!< heterogeneous player type id
+
+    char view_quality_; //!< view quality indicator, 'l' or 'h'
+
+    char focus_side_; //!< focus target side. 'l', 'r' or 'n'. 'n' means no focus target.
+    Int16 focus_unum_; //!< focus target uniform number. 0 means no focus target.
+
+    Int32 state_; //!< state bit flags
+
+    float x_; //!< position x
+    float y_; //!< position y
+    float vx_; //!< velocity x
+    float vy_; //!< velocity y
+    float body_; //!< body direction (degree)
+    float neck_; //!< head direction relative to body (degree)
+    float point_x_; //!< arm pointing x
+    float point_y_; //!< arm pointing y
+
+    float view_width_; //!< view width (degree). high: value>0, low: value<0
+
+    float stamina_; //!< satamina value
+    float effort_; //!< effort value
+    float recovery_; //!< recovery value
+
+    UInt16 kick_count_; //!< kick command count
+    UInt16 dash_count_; //!< dash command count
+    UInt16 turn_count_; //!< turn command count
+    UInt16 catch_count_; //!< catch command count
+    UInt16 move_count_; //!< move command count
+    UInt16 turn_neck_count_; //!< turn_neck command count
+    UInt16 change_view_count_; //!< change_view command count
+    UInt16 say_count_; //!< say command count
+    UInt16 tackle_count_; //!< tackle command count
+    UInt16 pointto_count_; //!< pointto command count
+    UInt16 attentionto_count_; //!< attentionto command count
+
+    /*!
+      \brief initialize all variables
+     */
+    PlayerT()
+        : side_( 'n' )
+        , unum_( 0 )
+        , type_( -1 )
+        , view_quality_( 'h' )
+        , focus_side_( 'n' )
+        , focus_unum_( 0 )
+        , state_( 0 )
+        , x_( 0.0f )
+        , y_( 0.0f )
+        , vx_( 0.0f )
+        , vy_( 0.0f )
+        , body_( 0.0f )
+        , neck_( 0.0f )
+        , point_x_( SHOWINFO_SCALE2F )
+        , point_y_( SHOWINFO_SCALE2F )
+        , view_width_( 0.0f )
+        , stamina_( -1.0f )
+        , effort_( 0.0f )
+        , recovery_( 0.0f )
+        , kick_count_( 0 )
+        , dash_count_( 0 )
+        , turn_count_( 0 )
+        , catch_count_( 0 )
+        , move_count_( 0 )
+        , turn_neck_count_( 0 )
+        , change_view_count_( 0 )
+        , say_count_( 0 )
+        , tackle_count_( 0 )
+        , pointto_count_( 0 )
+        , attentionto_count_( 0 )
+      { }
+
+    /*!
+      \brief get side id
+      \return side id
+     */
+    SideID side() const
+      {
+          return ( side_ == 'l' ? LEFT
+                   : side_ == 'r' ? RIGHT
+                   : NEUTRAL );
+      }
+
+    bool highQuality() const
+      {
+          return ( view_quality_ == 'h' );
+      }
+
+    /*!
+      \brief get focused player's side id
+      \return side id
+     */
+    SideID focusSide() const
+      {
+          return ( side_ == 'l' ? LEFT
+                   : side_ == 'r' ? RIGHT
+                   : NEUTRAL );
+      }
+};
+
+/*!
+  \struct TeamT
+  \brief team information
+ */
+struct TeamT {
+    std::string name_; //!< team name string
+    UInt16 score_; //!< total scores in normal game
+    UInt16 pen_score_; //!< count of penalty score
+    UInt16 pen_miss_; //!< count of penalty miss
+
+    /*!
+      \brief initialize all variables by 0
+     */
+    TeamT()
+        : score_( 0 )
+        , pen_score_( 0 )
+        , pen_miss_( 0 )
+      { }
+
+    /*!
+      \brief initialize all variables by specified values
+      \param name team name
+      \param score total score
+      \param pen_score count of penalty score
+      \param pen_miss count of penalty miss
+     */
+    TeamT( const char * name,
+           const UInt16 score,
+           const UInt16 pen_score,
+           const UInt16 pen_miss )
+        : name_( name )
+        , score_( score )
+        , pen_score_( pen_score )
+        , pen_miss_( pen_miss )
+      { }
+
+    /*!
+      \brief clear all variables
+     */
+    void clear()
+      {
+          name_.erase();
+          score_ = 0;
+          pen_score_ = 0;
+          pen_miss_ = 0;
+      }
+
+    /*!
+      \brief get the total penalty trial count
+      \return the total penalty trial count
+     */
+    int penaltyTrial() const
+      {
+          return pen_score_ + pen_miss_;
+      }
+
+    /*!
+      \brief check if artument variable has same values.
+      \param other compared variable.
+      \return compared result.
+     */
+    bool equals( const TeamT & other ) const
+      {
+          return ( name_ == other.name_
+                   && score_ == other.score_
+                   && pen_score_ == other.pen_score_
+                   && pen_miss_ == other.pen_miss_ );
+      }
+
+};
+
+/*!
+  \struct ShowInfoT
+  \brief show information
+ */
+struct ShowInfoT {
+    UInt32 time_;
+    //    PlayMode pmode_;
+    //    TeamT team_[2];
+    BallT ball_;
+    PlayerT player_[MAX_PLAYER * 2];
+};
+
+
+/*!
+  \struct DispInfoT
+  \brief display information
+ */
+struct DispInfoT {
+    PlayMode pmode_;
+    TeamT team_[2];
+    ShowInfoT show_;
+};
+
+
+//! recorded value of rcg v4
+const int REC_VERSION_4 = 4;
+
 //! default rcg version
-const int DEFAULT_LOG_VERSION = REC_VERSION_3;
+const int DEFAULT_LOG_VERSION = REC_VERSION_4;
 
 } // end namespace
 } // end namespace
