@@ -59,6 +59,8 @@ private:
     //! internal implementation object
     boost::scoped_ptr< TrainerAgentImpl > M_impl;
 
+protected:
+
     //! configuration parameter set
     TrainerConfig M_config;
 
@@ -378,7 +380,7 @@ public:
     */
     bool doChangePlayerType( const std::string & teamname,
                              const int unum,
-                             const HeteroID type );
+                             const int type );
 
     /*!
       \brief send say command (coach language)
