@@ -35,6 +35,8 @@
 
 #include "trainer_command.h"
 
+#include <rcsc/common/player_param.h>
+
 namespace rcsc {
 
 /*-------------------------------------------------------------------*/
@@ -361,7 +363,7 @@ TrainerMovePlayerCommand::toStr( std::ostream & to ) const
 
     if ( M_angle == -360.0 )
     {
-        return to;
+        return to << ")";
     }
 
     to << " " << M_angle;
@@ -391,7 +393,7 @@ TrainerRecoverCommand::toStr( std::ostream & to ) const
 TrainerChangePlayerTypeCommand::
 TrainerChangePlayerTypeCommand( const std::string & team_name,
                                 const int unum,
-                                const HeteroID type )
+                                const int type )
     : M_team_name( team_name )
     , M_unum( unum )
     , M_type( type )
@@ -406,8 +408,8 @@ TrainerChangePlayerTypeCommand( const std::string & team_name,
 std::ostream &
 TrainerChangePlayerTypeCommand::toStr( std::ostream & to ) const
 {
-    if ( M_type == Hetero_Unknown
-         || M_type == Hetero_MAX )
+    if ( M_type < Hetero_Unknown
+         || PlayerParam::i().playerTypes() <= M_type )
     {
         std::cerr << "TranierChangePlayerTypeCommand: illegal type Id"
                   << M_type
