@@ -21,30 +21,30 @@
 
 // first line
 (Init (goal_width 14.02) (player_size 0.299988)
-      (ball_size 0.0849915) (kickable_margin 0.699997)
-      (visible_distance 3) (kickable_area 1.08498)
-      (catchable_area_l 2) (catchable_area_w 1)
-      (half_time 3000) (ckick_margin 1)
-      (offside_active_area_size 2.5)
-      (offside_kick_margin 9.14999)
-      (audio_cut_dist 50))
+(ball_size 0.0849915) (kickable_margin 0.699997)
+(visible_distance 3) (kickable_area 1.08498)
+(catchable_area_l 2) (catchable_area_w 1)
+(half_time 3000) (ckick_margin 1)
+(offside_active_area_size 2.5)
+(offside_kick_margin 9.14999)
+(audio_cut_dist 50))
 
 (Info (state <time> <playmode> <score_l> <score_r>)
-      (ball <x> <y> <vx> <vy>)
-      (player {l|r} <unum>[ g]
-             (position <x> <y>
-                       <vx> <vy>
-                       <body>   // global body angle (degree)
-                       <head>   // global head angle (degree)
-             )
-             (stamina <sta>)
-             { (kick[ fault]) | (dash) | (turn) |
-               (catch[ fault]) | (move) | (tackle[ fault]) |
-               (none) }
-             [(say)]
-             [(turn_neck)]
-             [(change_view)]
-       )
+(ball <x> <y> <vx> <vy>)
+(player {l|r} <unum>[ g]
+(position <x> <y>
+<vx> <vy>
+<body>   // global body angle (degree)
+<head>   // global head angle (degree)
+)
+(stamina <sta>)
+{ (kick[ fault]) | (dash) | (turn) |
+(catch[ fault]) | (move) | (tackle[ fault]) |
+(none) }
+[(say)]
+[(turn_neck)]
+[(change_view)]
+)
 )
 
 // last line
@@ -87,6 +87,17 @@ private:
               move_ = rcsc::rcg::nstohi( player.move_count );
               change_view_ = rcsc::rcg::nstohi( player.change_view_count );
           }
+        void update( const rcsc::rcg::PlayerT & player )
+          {
+              kick_ = player.kick_count_;
+              dash_ = player.dash_count_;
+              turn_ = player.turn_count_;
+              say_ = player.say_count_;
+              turn_neck_ = player.turn_neck_count_;
+              catch_ = player.catch_count_;
+              move_ = player.move_count_;
+              change_view_ = player.change_view_count_;
+          }
     };
 
 
@@ -108,40 +119,40 @@ public:
     explicit
     TextPrinter( std::ostream & os );
 
-    virtual
+    // v3 or older
     bool handleDispInfo( const rcsc::rcg::dispinfo_t & disp );
-
-    virtual
     bool handleShowInfo( const rcsc::rcg::showinfo_t & show );
-
-    virtual
     bool handleShortShowInfo2( const rcsc::rcg::short_showinfo_t2 & show );
-
-    virtual
-    bool handleMsgInfo( short, const std::string & msg )
+    bool handleMsgInfo( rcsc::rcg::Int16,
+                        const std::string & msg )
       {
           return true;
       }
-
-    virtual
     bool handlePlayMode( char playmode );
-
-    virtual
     bool handleTeamInfo( const rcsc::rcg::team_t & team_left,
                          const rcsc::rcg::team_t & team_right );
-
-    virtual
     bool handlePlayerType( const rcsc::rcg::player_type_t & param );
-
-    virtual
     bool handleServerParam( const rcsc::rcg::server_params_t & param );
-
-    virtual
     bool handlePlayerParam( const rcsc::rcg::player_params_t & param );
 
-    virtual
+    // common
     bool handleEOF();
 
+    // v4 or later
+    bool handleShow( const int time,
+                     const rcsc::rcg::ShowInfoT & show );
+    bool handleMsg( const int time,
+                    const int board,
+                    const char * msg );
+    bool handlePlayMode( const int time,
+                         const rcsc::PlayMode pm );
+    bool handleTeam( const int time,
+                     const rcsc::rcg::TeamT & team_l,
+                     const rcsc::rcg::TeamT & team_r );
+    bool handleServerParam( const std::string & msg );
+    bool handlePlayerParam( const std::string & msg );
+    bool handlePlayerType( const std::string & msg );
+
 private:
     const
     std::string & getPlayModeString( const rcsc::PlayMode playmode ) const;
@@ -172,7 +183,7 @@ private:
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 TextPrinter::TextPrinter( std::ostream & os )
     : M_os( os )
     , M_init_written( false )
@@ -188,7 +199,7 @@ TextPrinter::TextPrinter( std::ostream & os )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 TextPrinter::handleDispInfo( const rcsc::rcg::dispinfo_t & disp )
 {
@@ -236,7 +247,7 @@ TextPrinter::handleDispInfo( const rcsc::rcg::dispinfo_t & disp )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 TextPrinter::handleShowInfo( const rcsc::rcg::showinfo_t & show )
 {
@@ -285,7 +296,7 @@ TextPrinter::handleShowInfo( const rcsc::rcg::showinfo_t & show )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 TextPrinter::handleShortShowInfo2( const rcsc::rcg::short_showinfo_t2 & show )
 {
@@ -331,7 +342,7 @@ TextPrinter::handleShortShowInfo2( const rcsc::rcg::short_showinfo_t2 & show )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 TextPrinter::handlePlayMode( char playmode )
 {
@@ -342,10 +353,10 @@ TextPrinter::handlePlayMode( char playmode )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
-TextPrinter::handleTeamInfo( const rcsc::rcg::team_t& team_left,
-                             const rcsc::rcg::team_t& team_right )
+TextPrinter::handleTeamInfo( const rcsc::rcg::team_t & team_left,
+                             const rcsc::rcg::team_t & team_right )
 {
     if ( M_left_team_name.empty() )
     {
@@ -371,7 +382,7 @@ TextPrinter::handleTeamInfo( const rcsc::rcg::team_t& team_left,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 TextPrinter::handlePlayerType( const rcsc::rcg::player_type_t & param )
 {
@@ -382,7 +393,7 @@ TextPrinter::handlePlayerType( const rcsc::rcg::player_type_t & param )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 TextPrinter::handleServerParam( const rcsc::rcg::server_params_t & param )
 {
@@ -412,7 +423,7 @@ TextPrinter::handleServerParam( const rcsc::rcg::server_params_t & param )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 TextPrinter::handlePlayerParam( const rcsc::rcg::player_params_t & param )
 {
@@ -423,7 +434,7 @@ TextPrinter::handlePlayerParam( const rcsc::rcg::player_params_t & param )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 TextPrinter::handleEOF()
 {
@@ -439,7 +450,133 @@ TextPrinter::handleEOF()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
+bool
+TextPrinter::handleShow( const int time,
+                         const rcsc::rcg::ShowInfoT & show )
+{
+    if ( ! M_init_written )
+    {
+        M_init_written = true;
+        M_os << "(Init)" << "\n";
+    }
+
+    M_os << "(Info ";
+    printState( M_os, show.time_ );
+
+    // ball
+    M_os << " (ball "
+         << show.ball_.x_ << ' ' << show.ball_.y_ << ' '
+         << show.ball_.vx_ << ' ' << show.ball_.vy_ << ')';
+
+    // players
+    for ( int i = 0; i < rcsc::MAX_PLAYER*2; ++i )
+    {
+        rcsc::rcg::player_t p;
+        rcsc::rcg::Serializer::convert( show.player_[i], p );
+
+        printPlayer( M_os,
+                     show.player_[i].side(),
+                     show.player_[i].unum_,
+                     M_command_count[i],
+                     p );
+        M_command_count[i].update( show.player_[i] );
+    }
+
+    M_os << ")\n";
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+TextPrinter::handleMsg( const int time,
+                        const int board,
+                        const char * msg )
+{
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+TextPrinter::handlePlayMode( const int time,
+                             const rcsc::PlayMode pm )
+{
+    M_playmode = pm;
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+TextPrinter::handleTeam( const int time,
+                         const rcsc::rcg::TeamT & team_l,
+                         const rcsc::rcg::TeamT & team_r )
+{
+    if ( M_left_team_name.empty() )
+    {
+        M_left_team_name = team_l.name_;
+    }
+    if ( M_right_team_name.empty() )
+    {
+        M_right_team_name = team_r.name_;
+    }
+
+    M_left_score = team_l.score_;
+    M_right_score = team_r.score_;
+
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+TextPrinter::handleServerParam( const std::string & msg )
+{
+    std::string::size_type pos = msg.find_first_of( ' ' );
+    if ( pos != std::string::npos )
+    {
+        M_init_written = true;
+        M_os << "(Init"
+             << msg.substr( pos )
+             << '\n';
+    }
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+TextPrinter::handlePlayerParam( const std::string & msg )
+{
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+TextPrinter::handlePlayerType( const std::string & msg )
+{
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
 const
 std::string &
 TextPrinter::getPlayModeString( const rcsc::PlayMode playmode ) const
@@ -511,7 +648,7 @@ TextPrinter::getPlayModeString( const rcsc::PlayMode playmode ) const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 std::ostream &
 TextPrinter::printState( std::ostream & os,
                          const long & cycle ) const
@@ -542,7 +679,7 @@ TextPrinter::printBall( std::ostream & os,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 std::ostream &
 TextPrinter::printBall( std::ostream & os,
                         const rcsc::rcg::ball_t & ball ) const
@@ -619,7 +756,7 @@ TextPrinter::printPlayer( std::ostream & os,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 std::ostream &
 TextPrinter::printPlayer( std::ostream & os,
                           //const std::string & teamname,
